@@ -50,32 +50,32 @@ pub struct BartVocabResources;
 pub struct BartMergesResources;
 
 impl BartModelResources {
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART: (&'static str, &'static str) = (
         "bart/model",
         "https://huggingface.co/facebook/bart-large/resolve/main/rust_model.ot",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_CNN: (&'static str, &'static str) = (
         "bart-cnn/model",
         "https://huggingface.co/facebook/bart-large-cnn/resolve/main/rust_model.ot",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_XSUM: (&'static str, &'static str) = (
         "bart-xsum/model",
         "https://huggingface.co/facebook/bart-large-xsum/resolve/main/rust_model.ot",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_MNLI: (&'static str, &'static str) = (
         "bart-large-mnli/model",
         "https://huggingface.co/facebook/bart-large-mnli/resolve/main/rust_model.ot",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-6-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-6-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_6_6: (&'static str, &'static str) = (
         "distilbart-cnn-6-6/model",
         "https://huggingface.co/sshleifer/distilbart-cnn-6-6/resolve/main/rust_model.ot",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-12-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-12-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_12_6: (&'static str, &'static str) = (
         "distilbart-cnn-12-6/model",
         "https://huggingface.co/sshleifer/distilbart-cnn-12-6/resolve/main/rust_model.ot",
@@ -83,32 +83,32 @@ impl BartModelResources {
 }
 
 impl BartConfigResources {
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART: (&'static str, &'static str) = (
         "bart/config",
         "https://huggingface.co/facebook/bart-large/resolve/main/config.json",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_CNN: (&'static str, &'static str) = (
         "bart-cnn/config",
         "https://huggingface.co/facebook/bart-large-cnn/resolve/main/config.json",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_XSUM: (&'static str, &'static str) = (
         "bart-xsum/config",
         "https://huggingface.co/facebook/bart-large-xsum/resolve/main/config.json",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_MNLI: (&'static str, &'static str) = (
         "bart-large-mnli/config",
         "https://huggingface.co/facebook/bart-large-mnli/resolve/main/config.json",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-6-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-6-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_6_6: (&'static str, &'static str) = (
         "distilbart-cnn-6-6/config",
         "https://cdn.huggingface.co/sshleifer/distilbart-cnn-6-6/config.json",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-12-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-12-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_12_6: (&'static str, &'static str) = (
         "distilbart-cnn-12-6/config",
         "https://cdn.huggingface.co/sshleifer/distilbart-cnn-12-6/config.json",
@@ -116,32 +116,32 @@ impl BartConfigResources {
 }
 
 impl BartVocabResources {
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART: (&'static str, &'static str) = (
         "bart/vocab",
         "https://huggingface.co/roberta-large/resolve/main/vocab.json",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_CNN: (&'static str, &'static str) = (
         "bart-cnn/vocab",
         "https://huggingface.co/roberta-large/resolve/main/vocab.json",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_XSUM: (&'static str, &'static str) = (
         "bart-xsum/vocab",
         "https://huggingface.co/roberta-large/resolve/main/vocab.json",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_MNLI: (&'static str, &'static str) = (
         "bart-large-mnli/vocab",
         "https://huggingface.co/roberta-large/resolve/main/vocab.json",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-6-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-6-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_6_6: (&'static str, &'static str) = (
         "distilbart-cnn-6-6/vocab",
         "https://cdn.huggingface.co/sshleifer/distilbart-cnn-6-6/vocab.json",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-12-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-12-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_12_6: (&'static str, &'static str) = (
         "distilbart-cnn-12-6/vocab",
         "https://cdn.huggingface.co/sshleifer/distilbart-cnn-12-6/vocab.json",
@@ -149,32 +149,32 @@ impl BartVocabResources {
 }
 
 impl BartMergesResources {
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART: (&'static str, &'static str) = (
         "bart/merges",
         "https://huggingface.co/roberta-large/resolve/main/merges.txt",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_CNN: (&'static str, &'static str) = (
         "bart-cnn/merges",
         "https://huggingface.co/roberta-large/resolve/main/merges.txt",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_XSUM: (&'static str, &'static str) = (
         "bart-xsum/merges",
         "https://huggingface.co/roberta-large/resolve/main/merges.txt",
     );
-    /// Shared under MIT license by the Facebook AI Research Fairseq team at https://github.com/pytorch/fairseq. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Facebook AI Research Fairseq team at <https://github.com/pytorch/fairseq>. Modified with conversion to C-array format.
     pub const BART_MNLI: (&'static str, &'static str) = (
         "bart-large-mnli/merges",
         "https://huggingface.co/roberta-large/resolve/main/merges.txt",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-6-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-6-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_6_6: (&'static str, &'static str) = (
         "distilbart-cnn-6-6/merges",
         "https://cdn.huggingface.co/sshleifer/distilbart-cnn-6-6/merges.txt",
     );
-    /// Shared under Apache 2.0 license by the Hugging Face team at https://huggingface.co/sshleifer/distilbart-cnn-12-6. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Hugging Face team at <https://huggingface.co/sshleifer/distilbart-cnn-12-6>. Modified with conversion to C-array format.
     pub const DISTILBART_CNN_12_6: (&'static str, &'static str) = (
         "distilbart-cnn-12-6/merges",
         "https://cdn.huggingface.co/sshleifer/distilbart-cnn-12-6/merges.txt",
@@ -222,9 +222,9 @@ pub struct BartConfig {
     pub vocab_size: i64,
 }
 
-impl Config<BartConfig> for BartConfig {}
+impl Config for BartConfig {}
 
-fn _make_causal_mask(
+pub(crate) fn _make_causal_mask(
     input_ids_shape: &[i64],
     dtype: Kind,
     device: Device,
@@ -240,7 +240,7 @@ fn _make_causal_mask(
     );
     let mask_cond = Tensor::arange(target_length, (dtype, device));
     let _ = mask.masked_fill_(
-        &mask_cond.lt1(&(&mask_cond + 1).view([target_length, 1])),
+        &mask_cond.lt_tensor(&(&mask_cond + 1).view([target_length, 1])),
         0,
     );
 
@@ -306,7 +306,10 @@ pub(crate) fn _prepare_decoder_attention_mask(
 }
 
 fn _shift_tokens_right(input_ids: &Tensor, pad_token_id: i64) -> Tensor {
-    let index_eos: Tensor = input_ids.ne(pad_token_id).sum1(&[-1], true, Int64) - 1;
+    let index_eos: Tensor = input_ids
+        .ne(pad_token_id)
+        .sum_dim_intlist(&[-1], true, Int64)
+        - 1;
     let output = input_ids.empty_like().to_kind(Int64);
     output
         .select(1, 0)
@@ -388,7 +391,7 @@ impl BartModel {
     ///
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *source_sequence_length*). Must be provided when not running in generation mode
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *source_sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `encoder_outputs` - Optional tuple made of a tensor of shape (*batch size*, *source_sequence_length*, *encoder_hidden_dim*) and optional vectors of tensors of length *num_encoder_layers* with shape (*batch size*, *source_sequence_length*, *hidden_size*).
     /// These correspond to the encoder last hidden state and optional hidden states/attention weights for encoder layers. When provided, the encoder hidden state will not be recalculated. Useful for generation tasks.
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
@@ -483,8 +486,8 @@ impl BartModel {
         let encoder_output = encoder_output.unwrap_or_else(|| calc_hidden_states.as_ref().unwrap());
 
         let decoder_output = self.decoder.forward_t(
-            &decoder_input_ids,
-            &encoder_output,
+            decoder_input_ids,
+            encoder_output,
             attention_mask,
             decoder_attention_mask,
             &self.embeddings,
@@ -551,7 +554,7 @@ impl BartForConditionalGeneration {
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *source_sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
     /// * `encoder_outputs` - Optional tuple made of a tensor of shape (*batch size*, *source_sequence_length*, *encoder_hidden_dim*) and optional vectors of tensors of length *num_encoder_layers* with shape (*batch size*, *source_sequence_length*, *hidden_size*).
     /// These correspond to the encoder last hidden state and optional hidden states/attention weights for encoder layers. When provided, the encoder hidden state will not be recalculated. Useful for generation tasks.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
     /// * `train` - boolean flag to turn on/off the dropout layers in the model. Should be set to false for inference.
     ///
@@ -745,7 +748,7 @@ impl BartForSequenceClassification {
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *source_sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
     /// * `encoder_outputs` - Optional tuple made of a tensor of shape (*batch size*, *source_sequence_length*, *encoder_hidden_dim*) and optional vectors of tensors of length *num_encoder_layers* with shape (*batch size*, *source_sequence_length*, *hidden_size*).
     /// These correspond to the encoder last hidden state and optional hidden states/attention weights for encoder layers. When provided, the encoder hidden state will not be recalculated. Useful for generation tasks.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
     /// * `train` - boolean flag to turn on/off the dropout layers in the model. Should be set to false for inference.
     ///
@@ -767,13 +770,13 @@ impl BartForSequenceClassification {
     /// # use rust_bert::Config;
     /// # use std::path::Path;
     /// # use tch::kind::Kind::{Int64, Double};
-    /// use rust_bert::bart::{BartConfig, BartForConditionalGeneration};
+    /// use rust_bert::bart::{BartConfig, BartForSequenceClassification};
     /// # let config_path = Path::new("path/to/config.json");
     /// # let vocab_path = Path::new("path/to/vocab.txt");
     /// # let device = Device::Cpu;
     /// # let vs = nn::VarStore::new(device);
     /// # let config = BartConfig::from_file(config_path);
-    /// # let bart_model: BartForConditionalGeneration = BartForConditionalGeneration::new(&vs.root(), &config);
+    /// # let bart_model: BartForSequenceClassification = BartForSequenceClassification::new(&vs.root(), &config);
     ///  let (batch_size, source_sequence_length, target_sequence_length) = (64, 128, 56);
     ///  let input_tensor = Tensor::rand(&[batch_size, source_sequence_length], (Int64, device));
     ///  let target_tensor = Tensor::rand(&[batch_size, target_sequence_length], (Int64, device));
@@ -782,12 +785,11 @@ impl BartForSequenceClassification {
     ///
     ///  let model_output = no_grad(|| {
     ///    bart_model
-    ///         .forward_t(Some(&input_tensor),
+    ///         .forward_t(&input_tensor,
     ///                    Some(&encoder_attention_mask),
     ///                    None,
     ///                    Some(&target_tensor),
     ///                    Some(&decoder_attention_mask),
-    ///                    None,
     ///                    false)
     ///    });
     /// ```
@@ -810,7 +812,7 @@ impl BartForSequenceClassification {
             train,
         );
         let eos_mask = input_ids.eq(self.eos_token_id);
-        let reshape = eos_mask.sum1(&[1], true, Int64);
+        let reshape = eos_mask.sum_dim_intlist(&[1], true, Int64);
         let sentence_representation = base_model_output
             .decoder_output
             .permute(&[2, 0, 1])
@@ -845,7 +847,7 @@ impl LMHeadModel for BartForConditionalGeneration {
     /// # Arguments
     ///
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *sequence_length*). If None, pre-computed embeddings must be provided (see `input_embeds`)
-    /// * `layer_past` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing th elast calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
+    /// * `layer_past` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing the last calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
     /// * `attention_mask` - Optional mask of shape (*batch size*, *sequence_length*). Masked position have value 0, non-masked value 1. If None set to 1
     /// * `input_embeds` - Unused for BART
     /// * `token_type_ids` - Unused for BART
@@ -896,21 +898,21 @@ impl LMHeadModel for BartForConditionalGeneration {
     /// ```
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         cache: Cache,
-        attention_mask: &Option<Tensor>,
-        _token_type_ids: &Option<Tensor>,
-        _position_ids: &Option<Tensor>,
-        _input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        _token_type_ids: Option<&Tensor>,
+        _position_ids: Option<&Tensor>,
+        _input_embeds: Option<&Tensor>,
         encoder_outputs: Option<&Tensor>,
-        decoder_input_ids: &Option<Tensor>,
+        decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let base_model_output = match cache {
             Cache::BARTCache(cached_layer_states) => self.base_model.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
-                decoder_input_ids.as_ref(),
+                input_ids,
+                attention_mask,
+                decoder_input_ids,
                 encoder_outputs,
                 None,
                 cached_layer_states,
@@ -918,9 +920,9 @@ impl LMHeadModel for BartForConditionalGeneration {
             ),
 
             Cache::None => self.base_model.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
-                decoder_input_ids.as_ref(),
+                input_ids,
+                attention_mask,
+                decoder_input_ids,
                 encoder_outputs,
                 None,
                 None,
@@ -1113,7 +1115,7 @@ impl PrivateLanguageGenerator<BartForConditionalGeneration, RobertaVocab, Robert
     fn get_model(&self) -> &BartForConditionalGeneration {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
@@ -1149,9 +1151,13 @@ impl PrivateLanguageGenerator<BartForConditionalGeneration, RobertaVocab, Robert
         scores: &mut Tensor,
         current_length: i64,
         max_length: i64,
+        forced_bos_token_id: Option<i64>,
     ) {
         if current_length == 1 {
-            self.force_token_id_generation(scores, &[self.get_bos_id().unwrap()]);
+            self.force_token_id_generation(
+                scores,
+                &[forced_bos_token_id.unwrap_or_else(|| self.get_bos_id().unwrap())],
+            );
         } else if current_length == max_length - 1 {
             self.force_token_id_generation(scores, self.get_eos_ids().as_ref().unwrap());
         }
@@ -1198,7 +1204,7 @@ impl PrivateLanguageGenerator<BartForConditionalGeneration, RobertaVocab, Robert
     where
         S: AsRef<[&'a str]>,
     {
-        let tokens = self.get_tokenizer().encode_list(
+        let tokens = self._get_tokenizer().encode_list(
             prompt_text.as_ref(),
             max_len as usize,
             &TruncationStrategy::LongestFirst,
@@ -1214,7 +1220,7 @@ impl PrivateLanguageGenerator<BartForConditionalGeneration, RobertaVocab, Robert
         let pad_token = match pad_token_id {
             Some(value) => value,
             None => self
-                .get_tokenizer()
+                ._get_tokenizer()
                 .convert_tokens_to_ids(&[RobertaVocab::unknown_value()])[0],
         };
 
