@@ -14,7 +14,7 @@
 
 use crate::common::activations::Activation;
 use crate::common::dropout::Dropout;
-use crate::common::linear::{linear_no_bias, LinearNoBias};
+use crate::common::embeddings::process_ids_embeddings_pair;
 use crate::gpt2::transformer::Block;
 use crate::pipelines::common::{ModelType, TokenizerOption};
 use crate::pipelines::generation_utils::private_generation_utils::{
@@ -45,32 +45,32 @@ pub struct Gpt2VocabResources;
 pub struct Gpt2MergesResources;
 
 impl Gpt2ModelResources {
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2: (&'static str, &'static str) = (
         "gpt2/model",
         "https://huggingface.co/gpt2/resolve/main/rust_model.ot",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_MEDIUM: (&'static str, &'static str) = (
         "gpt2-medium/model",
         "https://huggingface.co/gpt2-medium/resolve/main/rust_model.ot",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_LARGE: (&'static str, &'static str) = (
         "gpt2-large/model",
         "https://huggingface.co/gpt2-large/resolve/main/rust_model.ot",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_XL: (&'static str, &'static str) = (
         "gpt2-xl/model",
         "https://huggingface.co/gpt2-xl/resolve/main/rust_model.ot",
     );
-    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at https://huggingface.co/models. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at <https://huggingface.co/models>. Modified with conversion to C-array format.
     pub const DISTIL_GPT2: (&'static str, &'static str) = (
         "distilgpt2/model",
         "https://huggingface.co/distilgpt2/resolve/main/rust_model.ot",
     );
-    /// Shared under MIT license by the Microsoft team at https://huggingface.co/microsoft/DialoGPT-medium. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://huggingface.co/microsoft/DialoGPT-medium>. Modified with conversion to C-array format.
     pub const DIALOGPT_MEDIUM: (&'static str, &'static str) = (
         "dialogpt-medium/model",
         "https://huggingface.co/microsoft/DialoGPT-medium/resolve/main/rust_model.ot",
@@ -78,32 +78,32 @@ impl Gpt2ModelResources {
 }
 
 impl Gpt2ConfigResources {
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2: (&'static str, &'static str) = (
         "gpt2/config",
         "https://huggingface.co/gpt2/resolve/main/config.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_MEDIUM: (&'static str, &'static str) = (
         "gpt2-medium/config",
         "https://huggingface.co/gpt2-medium/resolve/main/config.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_LARGE: (&'static str, &'static str) = (
         "gpt2-large/config",
         "https://huggingface.co/gpt2-large/resolve/main/config.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_XL: (&'static str, &'static str) = (
         "gpt2-xl/config",
         "https://huggingface.co/gpt2-xl/resolve/main/config.json",
     );
-    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at https://huggingface.co/models. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at <https://huggingface.co/models>. Modified with conversion to C-array format.
     pub const DISTIL_GPT2: (&'static str, &'static str) = (
         "distilgpt2/config",
         "https://huggingface.co/distilgpt2/resolve/main/config.json",
     );
-    /// Shared under MIT license by the Microsoft team at https://huggingface.co/microsoft/DialoGPT-medium. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://huggingface.co/microsoft/DialoGPT-medium>. Modified with conversion to C-array format.
     pub const DIALOGPT_MEDIUM: (&'static str, &'static str) = (
         "dialogpt-medium/config",
         "https://huggingface.co/microsoft/DialoGPT-medium/resolve/main/config.json",
@@ -111,32 +111,32 @@ impl Gpt2ConfigResources {
 }
 
 impl Gpt2VocabResources {
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2: (&'static str, &'static str) = (
         "gpt2/vocab",
         "https://huggingface.co/gpt2/resolve/main/vocab.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_MEDIUM: (&'static str, &'static str) = (
         "gpt2-medium/vocab",
         "https://huggingface.co/gpt2-medium/resolve/main/vocab.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_LARGE: (&'static str, &'static str) = (
         "gpt2-large/vocab",
         "https://huggingface.co/gpt2-large/resolve/main/vocab.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_XL: (&'static str, &'static str) = (
         "gpt2-xl/vocab",
         "https://huggingface.co/gpt2-xl/resolve/main/vocab.json",
     );
-    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at https://huggingface.co/models. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at <https://huggingface.co/models>. Modified with conversion to C-array format.
     pub const DISTIL_GPT2: (&'static str, &'static str) = (
         "distilgpt2/vocab",
         "https://huggingface.co/distilgpt2/resolve/main/vocab.json",
     );
-    /// Shared under MIT license by the Microsoft team at https://huggingface.co/microsoft/DialoGPT-medium. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://huggingface.co/microsoft/DialoGPT-medium>. Modified with conversion to C-array format.
     pub const DIALOGPT_MEDIUM: (&'static str, &'static str) = (
         "dialogpt-medium/vocab",
         "https://huggingface.co/microsoft/DialoGPT-medium/resolve/main/vocab.json",
@@ -144,32 +144,32 @@ impl Gpt2VocabResources {
 }
 
 impl Gpt2MergesResources {
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2: (&'static str, &'static str) = (
         "gpt2/merges",
         "https://huggingface.co/gpt2/resolve/main/merges.txt",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_MEDIUM: (&'static str, &'static str) = (
         "gpt2-medium/merges",
         "https://huggingface.co/gpt2-medium/resolve/main/merges.txt",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_LARGE: (&'static str, &'static str) = (
         "gpt2-large/merges",
         "https://huggingface.co/gpt2-large/resolve/main/merges.txt",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT2_XL: (&'static str, &'static str) = (
         "gpt2-xl/merges",
         "https://huggingface.co/gpt2-xl/resolve/main/merges.txt",
     );
-    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at https://huggingface.co/models. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the HuggingFace Inc. team at <https://huggingface.co/models>. Modified with conversion to C-array format.
     pub const DISTIL_GPT2: (&'static str, &'static str) = (
         "distilgpt2/merges",
         "https://huggingface.co/distilgpt2/resolve/main/merges.txt",
     );
-    /// Shared under MIT license by the Microsoft team at https://huggingface.co/microsoft/DialoGPT-medium. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://huggingface.co/microsoft/DialoGPT-medium>. Modified with conversion to C-array format.
     pub const DIALOGPT_MEDIUM: (&'static str, &'static str) = (
         "dialogpt-medium/merges",
         "https://huggingface.co/microsoft/DialoGPT-medium/resolve/main/merges.txt",
@@ -200,7 +200,7 @@ pub struct Gpt2Config {
     pub vocab_size: i64,
 }
 
-impl Config<Gpt2Config> for Gpt2Config {}
+impl Config for Gpt2Config {}
 
 /// # GPT2 Base model
 /// Base architecture for GPT2 model. Usually complemented with a task-specific head, such as a language model head.
@@ -348,12 +348,12 @@ impl Gpt2Model {
     /// let model_output = no_grad(|| {
     ///     gpt2_model
     ///         .forward_t(
-    ///             &Some(input_tensor),
-    ///             &Some(past),
-    ///             &Some(attention_mask),
-    ///             &Some(token_type_ids),
-    ///             &Some(position_ids),
-    ///             &None,
+    ///             Some(&input_tensor),
+    ///             Some(&past),
+    ///             Some(&attention_mask),
+    ///             Some(&token_type_ids),
+    ///             Some(&position_ids),
+    ///             None,
     ///             false,
     ///         )
     ///         .unwrap()
@@ -361,35 +361,20 @@ impl Gpt2Model {
     /// ```
     pub fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
-        layer_past: &Option<Vec<Tensor>>,
-        attention_mask: &Option<Tensor>,
-        token_type_ids: &Option<Tensor>,
-        position_ids: &Option<Tensor>,
-        input_embeds: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
+        layer_past: Option<&Vec<Tensor>>,
+        attention_mask: Option<&Tensor>,
+        token_type_ids: Option<&Tensor>,
+        position_ids: Option<&Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> Result<Gpt2ModelOutput, RustBertError> {
-        let (input_embeddings, seq_length) = match input_ids {
-            Some(input_value) => match input_embeds {
-                Some(_) => {
-                    return Err(RustBertError::ValueError(
-                        "Only one of input ids or input embeddings may be set".into(),
-                    ));
-                }
-                None => (
-                    input_value.apply(&self.wte),
-                    *input_value.size().last().unwrap(),
-                ),
-            },
-            None => match input_embeds {
-                Some(embeds) => (embeds.copy(), embeds.size()[1]),
-                None => {
-                    return Err(RustBertError::ValueError(
-                        "At least one of input ids or input embeddings must be set".into(),
-                    ));
-                }
-            },
-        };
+        let (calc_input_embeddings, input_size, _) =
+            process_ids_embeddings_pair(input_ids, input_embeds, &self.wte)?;
+        let input_embeddings =
+            input_embeds.unwrap_or_else(|| calc_input_embeddings.as_ref().unwrap());
+
+        let seq_length = input_size[1];
 
         let (layer_past, layer_past_length) = match layer_past {
             Some(value) => {
@@ -415,7 +400,7 @@ impl Gpt2Model {
 
         let position_ids = match position_ids {
             Some(value) => value.copy(),
-            None => Tensor::arange1(
+            None => Tensor::arange_start(
                 layer_past_length,
                 seq_length + layer_past_length,
                 (Int64, input_embeddings.device()),
@@ -455,11 +440,8 @@ impl Gpt2Model {
         let layer_iter = self.h.iter().zip(layer_past);
         for layer_values in layer_iter {
             let (layer, past) = layer_values;
-            if let Some(hidden_states) = all_hidden_states.borrow_mut() {
-                hidden_states.push(hidden_state.as_ref().copy());
-            };
-
-            let temp = layer.forward_t(&hidden_state, &past, &attention_mask, train);
+            let temp =
+                layer.forward_t(&hidden_state, past.as_ref(), attention_mask.as_ref(), train);
             hidden_state = temp.0;
             if let Some(presents) = all_presents.borrow_mut() {
                 presents.push(temp.1.as_ref().copy());
@@ -467,6 +449,9 @@ impl Gpt2Model {
             if let Some(attentions) = all_attentions.borrow_mut() {
                 attentions.push(temp.2.as_ref().unwrap().copy());
             };
+            if let Some(hidden_states) = all_hidden_states.borrow_mut() {
+                hidden_states.push(hidden_state.as_ref().copy());
+            };
         }
 
         Ok(Gpt2ModelOutput {
@@ -482,10 +467,8 @@ impl Gpt2Model {
 /// GPT2 model with a decoding head (linear layer without bias). The weights of the linear layer are tied to the word embeddings
 /// It is made of the following blocks:
 /// - `transformer`: Base Gpt2Model
-/// - `lm_head`: Linear layer without bias tied to the weights of the token id embeddings
 pub struct GPT2LMHeadModel {
     transformer: Gpt2Model,
-    lm_head: LinearNoBias,
 }
 
 impl GPT2LMHeadModel {
@@ -517,16 +500,8 @@ impl GPT2LMHeadModel {
         let p = p.borrow();
 
         let transformer = Gpt2Model::new(p, config);
-        let lm_head = linear_no_bias(
-            p / "lm_head",
-            config.n_embd,
-            config.vocab_size,
-            Default::default(),
-        );
-        GPT2LMHeadModel {
-            transformer,
-            lm_head,
-        }
+
+        GPT2LMHeadModel { transformer }
     }
 }
 
@@ -590,14 +565,14 @@ impl LMHeadModel for GPT2LMHeadModel {
     /// let model_output = no_grad(|| {
     ///     gpt2_model
     ///         .forward_t(
-    ///             &Some(input_tensor),
+    ///             Some(&input_tensor),
     ///             Cache::GPT2Cache(Some(past)),
-    ///             &Some(attention_mask),
-    ///             &Some(token_type_ids),
-    ///             &Some(position_ids),
-    ///             &None,
+    ///             Some(&attention_mask),
+    ///             Some(&token_type_ids),
+    ///             Some(&position_ids),
+    ///             None,
+    ///             None,
     ///             None,
-    ///             &None,
     ///             false,
     ///         )
     ///         .unwrap()
@@ -605,20 +580,20 @@ impl LMHeadModel for GPT2LMHeadModel {
     /// ```
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         layer_past: Cache,
-        attention_mask: &Option<Tensor>,
-        token_type_ids: &Option<Tensor>,
-        position_ids: &Option<Tensor>,
-        input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        token_type_ids: Option<&Tensor>,
+        position_ids: Option<&Tensor>,
+        input_embeds: Option<&Tensor>,
         _encoder_outputs: Option<&Tensor>,
-        _decoder_input_ids: &Option<Tensor>,
+        _decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let base_model_output = match layer_past {
             Cache::GPT2Cache(layer_past) => self.transformer.forward_t(
                 input_ids,
-                &layer_past,
+                layer_past.as_ref(),
                 attention_mask,
                 token_type_ids,
                 position_ids,
@@ -627,7 +602,7 @@ impl LMHeadModel for GPT2LMHeadModel {
             ),
             Cache::None => self.transformer.forward_t(
                 input_ids,
-                &None,
+                None,
                 attention_mask,
                 token_type_ids,
                 position_ids,
@@ -641,7 +616,9 @@ impl LMHeadModel for GPT2LMHeadModel {
             }
         }?;
 
-        let lm_logits = base_model_output.output.apply(&self.lm_head);
+        let lm_logits = base_model_output
+            .output
+            .linear::<Tensor>(&self.transformer.wte.ws, None);
         Ok(LMModelOutput {
             lm_logits,
             cache: Cache::GPT2Cache(base_model_output.cache),
@@ -752,7 +729,7 @@ impl PrivateLanguageGenerator<GPT2LMHeadModel, Gpt2Vocab, Gpt2Tokenizer> for GPT
     fn get_model(&self) -> &GPT2LMHeadModel {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
