@@ -132,7 +132,9 @@ pub(crate) trait GptNeoAttentionUtils {
         let query_indices = Self::split_sequence_length_dim_to(&indices, num_blocks, block_length)?;
         let key_indices = Self::look_back(&indices, block_length, window_size, None, false)?;
 
-        let causal_mask = query_indices.unsqueeze(-1).ge1(&key_indices.unsqueeze(-2));
+        let causal_mask = query_indices
+            .unsqueeze(-1)
+            .ge_tensor(&key_indices.unsqueeze(-2));
 
         let calc_attention_mask = if attention_mask.is_none() {
             Some(Tensor::ones(
@@ -146,7 +148,7 @@ pub(crate) trait GptNeoAttentionUtils {
         let attention_mask =
             attention_mask.unwrap_or_else(|| calc_attention_mask.as_ref().unwrap());
         let attention_mask =
-            Self::look_back(&attention_mask, block_length, window_size, None, false)?.unsqueeze(-2);
+            Self::look_back(attention_mask, block_length, window_size, None, false)?.unsqueeze(-2);
         let causal_mask = causal_mask * attention_mask;
 
         let relative_position = key_indices.unsqueeze(-2) - query_indices.unsqueeze(-1);
@@ -212,7 +214,7 @@ pub(crate) trait GptNeoAttentionUtils {
     ) -> (Tensor, Tensor) {
         let mut attention_weights = query
             .matmul(&key.transpose(-1, -2))
-            .where1(causal_mask, &masked_bias.to_kind(query.kind()));
+            .where_self(causal_mask, &masked_bias.to_kind(query.kind()));
 
         if let Some(attention_mask_value) = attention_mask {
             attention_weights = attention_weights + attention_mask_value;
@@ -520,7 +522,7 @@ impl GptNeoLocalSelfAttention {
             &query,
             &key,
             &value,
-            &attention_mask,
+            attention_mask,
             &self.masked_bias,
             &self.attention_dropout,
             None,
