@@ -11,6 +11,7 @@
 // limitations under the License.
 
 use crate::common::dropout::Dropout;
+use crate::common::embeddings::process_ids_embeddings_pair;
 use crate::gpt_neo::attention::{GptNeoAttention, GptNeoAttentionUtils};
 use crate::gpt_neo::decoder::GptNeoBlock;
 use crate::gpt_neo::LayerState;
@@ -41,17 +42,17 @@ pub struct GptNeoVocabResources;
 pub struct GptNeoMergesResources;
 
 impl GptNeoModelResources {
-    /// Shared under Apache 2.0 license by the EleutherAI contributors at https://www.eleuther.ai. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_125M: (&'static str, &'static str) = (
         "gpt-neo-125M/model",
         "https://huggingface.co/EleutherAI/gpt-neo-125M/resolve/main/rust_model.ot",
     );
-    /// Shared under Apache 2.0 license by the EleutherAI contributors at https://www.eleuther.ai. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_1_3B: (&'static str, &'static str) = (
         "gpt-neo-1_3B/model",
         "https://huggingface.co/EleutherAI/gpt-neo-1.3B/resolve/main/rust_model.ot",
     );
-    /// Shared under Apache 2.0 license by the EleutherAI contributors at https://www.eleuther.ai. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_2_7B: (&'static str, &'static str) = (
         "gpt-neo-2_7B/model",
         "https://huggingface.co/EleutherAI/gpt-neo-2.7B/resolve/main/rust_model.ot",
@@ -59,17 +60,17 @@ impl GptNeoModelResources {
 }
 
 impl GptNeoConfigResources {
-    /// Shared under Apache 2.0 license by the EleutherAI contributors at https://www.eleuther.ai. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_125M: (&'static str, &'static str) = (
         "gpt-neo-125M/config",
         "https://huggingface.co/EleutherAI/gpt-neo-125M/resolve/main/config.json",
     );
-    /// Shared under Apache 2.0 license by the EleutherAI contributors at https://www.eleuther.ai. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_1_3B: (&'static str, &'static str) = (
         "gpt-neo-1_3B/config",
         "https://huggingface.co/EleutherAI/gpt-neo-1.3B/resolve/main/config.json",
     );
-    /// Shared under Apache 2.0 license by the EleutherAI contributors at https://www.eleuther.ai. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_2_7B: (&'static str, &'static str) = (
         "gpt-neo-2_7B/config",
         "https://huggingface.co/EleutherAI/gpt-neo-2.7B/resolve/main/config.json",
@@ -77,17 +78,17 @@ impl GptNeoConfigResources {
 }
 
 impl GptNeoVocabResources {
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT_NEO_125M: (&'static str, &'static str) = (
         "gpt-neo-125M/vocab",
         "https://huggingface.co/EleutherAI/gpt-neo-125M/resolve/main/vocab.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT_NEO_1_3B: (&'static str, &'static str) = (
         "gpt-neo-1_3B/vocab",
         "https://huggingface.co/EleutherAI/gpt-neo-1.3B/resolve/main/vocab.json",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Modified MIT license by the OpenAI team at <https://github.com/openai/gpt-2/blob/master/LICENSE>. Modified with conversion to C-array format.
     pub const GPT_NEO_2_7B: (&'static str, &'static str) = (
         "gpt-neo-2_7B/vocab",
         "https://huggingface.co/EleutherAI/gpt-neo-2.7B/resolve/main/vocab.json",
@@ -95,17 +96,17 @@ impl GptNeoVocabResources {
 }
 
 impl GptNeoMergesResources {
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_125M: (&'static str, &'static str) = (
         "gpt-neo-125M/merges",
         "https://huggingface.co/EleutherAI/gpt-neo-125M/resolve/main/merges.txt",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_1_3B: (&'static str, &'static str) = (
         "gpt-neo-1_3B/merges",
         "https://huggingface.co/EleutherAI/gpt-neo-1.3B/resolve/main/merges.txt",
     );
-    /// Shared under Modified MIT license by the OpenAI team at https://github.com/openai/gpt-2/blob/master/LICENSE. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the EleutherAI contributors at <https://www.eleuther.ai>. Modified with conversion to C-array format.
     pub const GPT_NEO_2_7B: (&'static str, &'static str) = (
         "gpt-neo-2_7B/merges",
         "https://huggingface.co/EleutherAI/gpt-neo-2.7B/resolve/main/merges.txt",
@@ -146,7 +147,7 @@ pub struct GptNeoConfig {
     pub resid_dropout: f64,
 }
 
-impl Config<GptNeoConfig> for GptNeoConfig {}
+impl Config for GptNeoConfig {}
 
 /// # GPT-Neo Base model
 /// Base architecture for GPT-Neo models. Task-specific models will be built from this common base model
@@ -263,7 +264,7 @@ impl GptNeoModel {
     ///   - `hidden_states` - `Tensor` of shape (*batch size*, *sequence_length*, *hidden_size*) representing the activations of the last hidden state
     ///   - `next_cache` - `Option<Vec<Option<LayerState>>>` of length *n_layer* containing the past content for the the attention layers
     ///   - `all_hidden_states` - `Option<Vec<Tensor>>` of length *n_layer + 1* with shape (*batch size*, *sequence_length*, *hidden_size*)
-    ///   - `all_attentions` - `Option<Vec<Tensor>>` of length *n_layer* containign the attention weights for each layer
+    ///   - `all_attentions` - `Option<Vec<Tensor>>` of length *n_layer* containing the attention weights for each layer
     ///
     /// # Example
     ///
@@ -305,27 +306,8 @@ impl GptNeoModel {
         attention_mask: Option<&Tensor>,
         train: bool,
     ) -> Result<GptNeoModelOutput, RustBertError> {
-        let (calc_input_embeddings, input_shape, device) = if let Some(input_ids) = input_ids {
-            if input_embeds.is_none() {
-                (
-                    Some(input_ids.apply(&self.word_embeddings)),
-                    input_ids.size(),
-                    input_ids.device(),
-                )
-            } else {
-                return Err(RustBertError::ValueError(
-                    "Only one of input ids or input embeddings may be set".into(),
-                ));
-            }
-        } else if let Some(input_embeds) = input_embeds {
-            let mut input_shape = input_embeds.size();
-            let _ = input_shape.pop();
-            (None, input_shape, input_embeds.device())
-        } else {
-            return Err(RustBertError::ValueError(
-                "At least one of input ids or input embeddings must be set".into(),
-            ));
-        };
+        let (calc_input_embeddings, input_shape, device) =
+            process_ids_embeddings_pair(input_ids, input_embeds, &self.word_embeddings)?;
 
         let (batch_size, current_sequence_length) = (input_shape[0], input_shape[1]);
 
@@ -345,7 +327,7 @@ impl GptNeoModel {
 
         let calc_position_ids = if position_ids.is_none() {
             let position_ids =
-                Tensor::arange1(past_length, full_sequence_length, (Kind::Int64, device));
+                Tensor::arange_start(past_length, full_sequence_length, (Kind::Int64, device));
             Some(
                 position_ids
                     .unsqueeze(0)
@@ -410,14 +392,8 @@ impl GptNeoModel {
             };
 
             let temp = if let Some(x_value) = &x {
-                if let Some(hidden_states) = all_hidden_states.borrow_mut() {
-                    hidden_states.push(x_value.copy());
-                }
                 layer.forward_t(x_value, layer_state.as_ref(), attention_mask, train)?
             } else {
-                if let Some(hidden_states) = all_hidden_states.borrow_mut() {
-                    hidden_states.push(hidden_state.copy());
-                }
                 layer.forward_t(&hidden_state, layer_state.as_ref(), attention_mask, train)?
             };
             x = Some(temp.0);
@@ -426,10 +402,10 @@ impl GptNeoModel {
             if let Some(attentions) = all_attentions.borrow_mut() {
                 attentions.push(attention_weights.as_ref().unwrap().copy());
             };
+            if let Some(hidden_states) = all_hidden_states.borrow_mut() {
+                hidden_states.push(x.as_ref().unwrap().copy());
+            };
         }
-        if let Some(hidden_states) = all_hidden_states.borrow_mut() {
-            hidden_states.push(x.as_ref().unwrap().copy());
-        };
 
         let hidden_states = x
             .unwrap()
@@ -504,7 +480,7 @@ impl GptNeoForCausalLM {
     ///   - `lm_logits` - `Tensor` of shape (*batch size*, *sequence_length*, *vocab_size*) representing the logits for each vocab item and position
     ///   - `next_cache` - `Option<Vec<Option<LayerState>>>` of length *n_layer* containing the past content for the the attention layers
     ///   - `all_hidden_states` - `Option<Vec<Tensor>>` of length *n_layer + 1* with shape (*batch size*, *sequence_length*, *hidden_size*)
-    ///   - `all_attentions` - `Option<Vec<Tensor>>` of length *n_layer* containign the attention weights for each layer
+    ///   - `all_attentions` - `Option<Vec<Tensor>>` of length *n_layer* containing the attention weights for each layer
     ///
     /// # Example
     ///
@@ -572,33 +548,33 @@ impl GptNeoForCausalLM {
 impl LMHeadModel for GptNeoForCausalLM {
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         layer_past: Cache,
-        attention_mask: &Option<Tensor>,
-        token_type_ids: &Option<Tensor>,
-        position_ids: &Option<Tensor>,
-        input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        token_type_ids: Option<&Tensor>,
+        position_ids: Option<&Tensor>,
+        input_embeds: Option<&Tensor>,
         _encoder_outputs: Option<&Tensor>,
-        _decoder_input_ids: &Option<Tensor>,
+        _decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let base_model_output = match layer_past {
             Cache::GPTNeoCache(layer_past) => self.forward_t(
-                input_ids.as_ref(),
-                input_embeds.as_ref(),
-                token_type_ids.as_ref(),
-                position_ids.as_ref(),
+                input_ids,
+                input_embeds,
+                token_type_ids,
+                position_ids,
                 layer_past,
-                attention_mask.as_ref(),
+                attention_mask,
                 train,
             ),
             Cache::None => self.forward_t(
-                input_ids.as_ref(),
-                input_embeds.as_ref(),
-                token_type_ids.as_ref(),
-                position_ids.as_ref(),
+                input_ids,
+                input_embeds,
+                token_type_ids,
+                position_ids,
                 None,
-                attention_mask.as_ref(),
+                attention_mask,
                 train,
             ),
             _ => {
@@ -729,7 +705,7 @@ impl PrivateLanguageGenerator<GptNeoForCausalLM, Gpt2Vocab, Gpt2Tokenizer> for G
     fn get_model(&self) -> &GptNeoForCausalLM {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
