@@ -58,6 +58,8 @@
 //! GPT-Neo| | | |✅ | | | |
 //! BART|✅| | |✅ |✅| | |
 //! Marian| | | |  | |✅| |
+//! MBart|✅| | |✅ | | | |
+//! M2M100| | | |✅ | | | |
 //! Electra | |✅| | | | |✅|
 //! ALBERT |✅|✅|✅| | | |✅|
 //! T5 | | | |✅ |✅|✅| |
@@ -79,7 +81,7 @@
 //!
 //! ### Manual installation (recommended)
 //!
-//! 1. Download `libtorch` from https://pytorch.org/get-started/locally/. This package requires `v1.8.1`: if this version is no longer available on the "get started" page,
+//! 1. Download `libtorch` from <https://pytorch.org/get-started/locally/>. This package requires `v1.8.1`: if this version is no longer available on the "get started" page,
 //! the file should be accessible by modifying the target link, for example `https://download.pytorch.org/libtorch/cu111/libtorch-shared-with-deps-1.8.1%2Bcu111.zip` for a Linux version with CUDA11.
 //! 2. Extract the library to a location of your choice
 //! 3. Set the following environment variables
@@ -145,44 +147,50 @@
 //! <details>
 //! <summary> <b>2. Translation </b> </summary>
 //!
-//! Translation using the MarianMT architecture and pre-trained models from the Opus-MT team from Language Technology at the University of Helsinki.
-//! Currently supported languages are :
-//!  - English <-> French
-//!  - English <-> Spanish
-//!  - English <-> Portuguese
-//!  - English <-> Italian
-//!  - English <-> Catalan
-//!  - English <-> German
-//!  - English <-> Russian
-//!  - English <-> Chinese (Simplified)
-//!  - English <-> Chinese (Traditional)
-//!  - English <-> Dutch
-//!  - English <-> Swedish
-//!  - English <-> Arabic
-//!  - English <-> Hebrew
-//!  - English <-> Hindi
-//!  - French <-> German
+//! Translation pipeline supporting a broad range of source and target languages. Leverages two main architectures for translation tasks:
+//! - Marian-based models, for specific source/target combinations
+//! - M2M100 models allowing for direct translation between 100 languages (at a higher computational cost and lower performance for some selected languages)
+//!
+//! Marian-based pretrained models for the following language pairs are readily available in the library - but the user can import any Pytorch-based
+//! model for predictions
+//! - English <-> French
+//! - English <-> Spanish
+//! - English <-> Portuguese
+//! - English <-> Italian
+//! - English <-> Catalan
+//! - English <-> German
+//! - English <-> Russian
+//! - English <-> Chinese
+//! - English <-> Dutch
+//! - English <-> Swedish
+//! - English <-> Arabic
+//! - English <-> Hebrew
+//! - English <-> Hindi
+//! - French <-> German
+//!
+//! For languages not supported by the proposed pretrained Marian models, the user can leverage a M2M100 model supporting direct translation between 100 languages (without intermediate English translation)
+//! The full list of supported languages is available in the [`pipelines` module](./pipelines/translation/enum.Language.html)
+//!
 //!
 //! ```no_run
-//! # fn main() -> anyhow::Result<()> {
-//! # use rust_bert::pipelines::generation_utils::LanguageGenerator;
-//! use rust_bert::pipelines::translation::{Language, TranslationConfig, TranslationModel};
-//! use tch::Device;
-//! let translation_config =
-//!     TranslationConfig::new(Language::EnglishToFrench, Device::cuda_if_available());
-//! let mut model = TranslationModel::new(translation_config)?;
-//!
-//! let input = ["This is a sentence to be translated"];
-//!
-//! let output = model.translate(&input);
-//! # Ok(())
-//! # }
+//! use rust_bert::pipelines::translation::{Language, TranslationModelBuilder};
+//! fn main() -> anyhow::Result<()> {
+//!     let model = TranslationModelBuilder::new()
+//!         .with_source_languages(vec![Language::English])
+//!         .with_target_languages(vec![Language::Spanish, Language::French, Language::Italian])
+//!         .create_model()?;
+//!     let input_text = "This is a sentence to be translated";
+//!     let output = model.translate(&[input_text], None, Language::Spanish)?;
+//!     for sentence in output {
+//!         println!("{}", sentence);
+//!     }
+//!     Ok(())
+//! }
 //! ```
-//!
 //! Output: \
 //! ```no_run
 //! # let output =
-//! "Il s'agit d'une phrase à traduire"
+//! " Il s'agit d'une phrase à traduire"
 //! # ;
 //! ```
 //!
@@ -578,7 +586,9 @@ pub mod electra;
 pub mod gpt2;
 pub mod gpt_neo;
 pub mod longformer;
+pub mod m2m_100;
 pub mod marian;
+pub mod mbart;
 pub mod mobilebert;
 pub mod openai_gpt;
 pub mod pegasus;
