@@ -352,7 +352,8 @@ impl LongformerSelfAttention {
         &self,
         is_index_global_attn: &Tensor,
     ) -> GlobalAttentionIndices {
-        let num_global_attention_indices = is_index_global_attn.sum1(&[1], false, Kind::Int64);
+        let num_global_attention_indices =
+            is_index_global_attn.sum_dim_intlist(&[1], false, Kind::Int64);
         let max_num_global_attention_indices = i64::from(num_global_attention_indices.max());
         let is_index_global_attn_nonzero = is_index_global_attn
             .nonzero_numpy()
@@ -364,7 +365,7 @@ impl LongformerSelfAttention {
             max_num_global_attention_indices,
             (Kind::Int64, is_index_global_attn.device()),
         )
-        .lt1(&num_global_attention_indices.unsqueeze(-1));
+        .lt_tensor(&num_global_attention_indices.unsqueeze(-1));
 
         let is_local_index_global_attention_nonzero = is_local_index_global_attention
             .nonzero_numpy()
@@ -422,13 +423,13 @@ impl LongformerSelfAttention {
         let _ = attention_probas_from_global_key
             .index_select(
                 0,
-                &is_local_index_no_global_attention_nonzero[0]
+                is_local_index_no_global_attention_nonzero[0]
                     .as_ref()
                     .unwrap(),
             )
             .index_select(
                 3,
-                &is_local_index_no_global_attention_nonzero[1]
+                is_local_index_no_global_attention_nonzero[1]
                     .as_ref()
                     .unwrap(),
             )
@@ -480,7 +481,7 @@ impl LongformerSelfAttention {
 
         let attn_output_without_global = self.sliding_chunks_matmul_attention_probas_value(
             &attention_probas_without_global,
-            &value_vectors,
+            value_vectors,
             self.one_sided_attention_window_size,
         );
         attention_output_only_global + attn_output_without_global
@@ -555,13 +556,13 @@ impl LongformerSelfAttention {
         let _ = global_attention_scores
             .index_select(
                 0,
-                &is_local_index_no_global_attention_nonzero[0]
+                is_local_index_no_global_attention_nonzero[0]
                     .as_ref()
                     .unwrap(),
             )
             .index_select(
                 2,
-                &is_local_index_no_global_attention_nonzero[1]
+                is_local_index_no_global_attention_nonzero[1]
                     .as_ref()
                     .unwrap(),
             )
