@@ -12,6 +12,7 @@
 
 use crate::common::activations::{Activation, TensorFunction};
 use crate::common::dropout::Dropout;
+use crate::common::embeddings::get_shape_and_device_from_ids_embeddings_pair;
 use crate::mobilebert::embeddings::MobileBertEmbeddings;
 use crate::mobilebert::encoder::{MobileBertEncoder, MobileBertPooler};
 use crate::{Config, RustBertError};
@@ -31,12 +32,12 @@ pub struct MobileBertConfigResources;
 pub struct MobileBertVocabResources;
 
 impl MobileBertModelResources {
-    /// Shared under Apache 2.0 license by the Google team at https://huggingface.co/google/mobilebert-uncased. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Google team at <https://huggingface.co/google/mobilebert-uncased>. Modified with conversion to C-array format.
     pub const MOBILEBERT_UNCASED: (&'static str, &'static str) = (
         "mobilebert-uncased/model",
         "https://huggingface.co/google/mobilebert-uncased/resolve/main/rust_model.ot",
     );
-    /// Shared under MIT license at https://huggingface.co/mrm8488/mobilebert-finetuned-pos. Modified with conversion to C-array format.
+    /// Shared under MIT license at <https://huggingface.co/mrm8488/mobilebert-finetuned-pos>. Modified with conversion to C-array format.
     pub const MOBILEBERT_ENGLISH_POS: (&'static str, &'static str) = (
         "mobilebert-finetuned-pos/model",
         "https://huggingface.co/mrm8488/mobilebert-finetuned-pos/resolve/main/rust_model.ot",
@@ -44,12 +45,12 @@ impl MobileBertModelResources {
 }
 
 impl MobileBertConfigResources {
-    /// Shared under Apache 2.0 license by the Google team at https://huggingface.co/google/mobilebert-uncased. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Google team at <https://huggingface.co/google/mobilebert-uncased>. Modified with conversion to C-array format.
     pub const MOBILEBERT_UNCASED: (&'static str, &'static str) = (
         "mobilebert-uncased/config",
         "https://huggingface.co/google/mobilebert-uncased/resolve/main/config.json",
     );
-    /// Shared under MIT license at https://huggingface.co/mrm8488/mobilebert-finetuned-pos. Modified with conversion to C-array format.
+    /// Shared under MIT license at <https://huggingface.co/mrm8488/mobilebert-finetuned-pos>. Modified with conversion to C-array format.
     pub const MOBILEBERT_ENGLISH_POS: (&'static str, &'static str) = (
         "mobilebert-finetuned-pos/config",
         "https://huggingface.co/mrm8488/mobilebert-finetuned-pos/resolve/main/config.json",
@@ -57,12 +58,12 @@ impl MobileBertConfigResources {
 }
 
 impl MobileBertVocabResources {
-    /// Shared under Apache 2.0 license by the Google team at https://huggingface.co/google/mobilebert-uncased. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Google team at <https://huggingface.co/google/mobilebert-uncased>. Modified with conversion to C-array format.
     pub const MOBILEBERT_UNCASED: (&'static str, &'static str) = (
         "mobilebert-uncased/vocab",
         "https://huggingface.co/google/mobilebert-uncased/resolve/main/vocab.txt",
     );
-    /// Shared under MIT license at https://huggingface.co/mrm8488/mobilebert-finetuned-pos. Modified with conversion to C-array format.
+    /// Shared under MIT license at <https://huggingface.co/mrm8488/mobilebert-finetuned-pos>. Modified with conversion to C-array format.
     pub const MOBILEBERT_ENGLISH_POS: (&'static str, &'static str) = (
         "mobilebert-finetuned-pos/vocab",
         "https://huggingface.co/mrm8488/mobilebert-finetuned-pos/resolve/main/vocab.txt",
@@ -194,7 +195,7 @@ pub struct MobileBertConfig {
     pub label2id: Option<HashMap<String, i64>>,
 }
 
-impl Config<MobileBertConfig> for MobileBertConfig {}
+impl Config for MobileBertConfig {}
 
 pub struct MobileBertPredictionHeadTransform {
     dense: nn::Linear,
@@ -319,7 +320,7 @@ impl MobileBertModel {
     ///
     /// * `p` - Variable store path for the root of the MobileBERT model
     /// * `config` - `MobileBertConfig` object defining the model architecture and decoder status
-    /// * `add_poling_layer` - boolean flag indicating if a pooling layer shuld be added after the encoder
+    /// * `add_poling_layer` - boolean flag indicating if a pooling layer should be added after the encoder
     ///
     /// # Example
     ///
@@ -418,28 +419,12 @@ impl MobileBertModel {
         input_ids: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         train: bool,
     ) -> Result<MobileBertOutput, RustBertError> {
-        let (input_shape, device) = match input_ids {
-            Some(input_value) => match &input_embeds {
-                Some(_) => {
-                    return Err(RustBertError::ValueError(
-                        "Only one of input ids or input embeddings may be set".into(),
-                    ));
-                }
-                None => (input_value.size(), input_value.device()),
-            },
-            None => match &input_embeds {
-                Some(embeds) => (vec![embeds.size()[0], embeds.size()[1]], embeds.device()),
-                None => {
-                    return Err(RustBertError::ValueError(
-                        "At least one of input ids or input embeddings must be set".into(),
-                    ));
-                }
-            },
-        };
+        let (input_shape, device) =
+            get_shape_and_device_from_ids_embeddings_pair(input_ids, input_embeds)?;
 
         let calc_attention_mask = if attention_mask.is_none() {
             Some(Tensor::ones(input_shape.as_slice(), (Kind::Int64, device)))
@@ -610,7 +595,7 @@ impl MobileBertForMaskedLM {
         input_ids: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         train: bool,
     ) -> Result<MobileBertMaskedLMOutput, RustBertError> {
@@ -752,7 +737,7 @@ impl MobileBertForSequenceClassification {
         input_ids: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         train: bool,
     ) -> Result<MobileBertSequenceClassificationOutput, RustBertError> {
@@ -885,7 +870,7 @@ impl MobileBertForQuestionAnswering {
         input_ids: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         train: bool,
     ) -> Result<MobileBertQuestionAnsweringOutput, RustBertError> {
@@ -901,8 +886,8 @@ impl MobileBertForQuestionAnswering {
         let sequence_output = mobilebert_output.hidden_state.apply(&self.qa_outputs);
         let logits = sequence_output.split(1, -1);
         let (start_logits, end_logits) = (&logits[0], &logits[1]);
-        let start_logits = start_logits.squeeze1(-1);
-        let end_logits = end_logits.squeeze1(-1);
+        let start_logits = start_logits.squeeze_dim(-1);
+        let end_logits = end_logits.squeeze_dim(-1);
 
         Ok(MobileBertQuestionAnsweringOutput {
             start_logits,
@@ -1022,7 +1007,7 @@ impl MobileBertForMultipleChoice {
         input_ids: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         train: bool,
     ) -> Result<MobileBertSequenceClassificationOutput, RustBertError> {
@@ -1052,7 +1037,7 @@ impl MobileBertForMultipleChoice {
             input_ids.as_ref(),
             token_type_ids.as_ref(),
             position_ids.as_ref(),
-            input_embeds,
+            input_embeds.as_ref(),
             attention_mask.as_ref(),
             train,
         )?;
@@ -1189,7 +1174,7 @@ impl MobileBertForTokenClassification {
         input_ids: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         train: bool,
     ) -> Result<MobileBertTokenClassificationOutput, RustBertError> {
