@@ -10,9 +10,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use crate::bart::{BartConfig, BartModelOutput};
+use crate::bart::BartModelOutput;
 use crate::common::resources::{RemoteResource, Resource};
 use crate::gpt2::{Gpt2ConfigResources, Gpt2ModelResources, Gpt2VocabResources};
+use crate::mbart::MBartConfig;
 use crate::pegasus::decoder::PegasusDecoder;
 use crate::pegasus::encoder::PegasusEncoder;
 use crate::pegasus::LayerState;
@@ -40,7 +41,7 @@ pub struct PegasusConfigResources;
 pub struct PegasusVocabResources;
 
 impl PegasusModelResources {
-    /// Shared under Apache 2.0 license by the Pegasus team at https://huggingface.co/google/pegasus-cnn_dailymail. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Pegasus team at <https://huggingface.co/google/pegasus-cnn_dailymail>. Modified with conversion to C-array format.
     pub const CNN_DAILYMAIL: (&'static str, &'static str) = (
         "pegasus-cnn_dailymail/model",
         "https://huggingface.co/google/pegasus-cnn_dailymail/resolve/main/rust_model.ot",
@@ -48,7 +49,7 @@ impl PegasusModelResources {
 }
 
 impl PegasusConfigResources {
-    /// Shared under Apache 2.0 license by the Pegasus team at https://huggingface.co/google/pegasus-cnn_dailymail.
+    /// Shared under Apache 2.0 license by the Pegasus team at <https://huggingface.co/google/pegasus-cnn_dailymail>.
     pub const CNN_DAILYMAIL: (&'static str, &'static str) = (
         "pegasus-cnn_dailymail/config",
         "https://huggingface.co/google/pegasus-cnn_dailymail/resolve/main/config.json",
@@ -56,7 +57,7 @@ impl PegasusConfigResources {
 }
 
 impl PegasusVocabResources {
-    /// Shared under Apache 2.0 license by the Pegasus team at https://huggingface.co/google/pegasus-cnn_dailymail.
+    /// Shared under Apache 2.0 license by the Pegasus team at <https://huggingface.co/google/pegasus-cnn_dailymail>.
     pub const CNN_DAILYMAIL: (&'static str, &'static str) = (
         "pegasus-cnn_dailymail/spiece",
         "https://huggingface.co/google/pegasus-cnn_dailymail/resolve/main/spiece.model",
@@ -65,7 +66,7 @@ impl PegasusVocabResources {
 
 /// # Pegasus model configuration
 /// Defines the Pegasus model architecture (e.g. number of layers, hidden layer size, label mapping...)
-pub type PegasusConfig = BartConfig;
+pub type PegasusConfig = MBartConfig;
 
 fn _shift_tokens_right(
     input_ids: &Tensor,
@@ -155,7 +156,7 @@ impl PegasusModel {
     ///
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *source_sequence_length*). Must be provided when not running in generation mode
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *source_sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `encoder_outputs` - Optional tuple made of a tensor of shape (*batch size*, *source_sequence_length*, *encoder_hidden_dim*) and optional vectors of tensors of length *num_encoder_layers* with shape (*batch size*, *source_sequence_length*, *hidden_size*).
     /// These correspond to the encoder last hidden state and optional hidden states/attention weights for encoder layers. When provided, the encoder hidden state will not be recalculated. Useful for generation tasks.
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
@@ -241,8 +242,8 @@ impl PegasusModel {
         let encoder_output = encoder_output.unwrap_or_else(|| calc_hidden_states.as_ref().unwrap());
 
         let decoder_output = self.decoder.forward_t(
-            &decoder_input_ids,
-            &encoder_output,
+            decoder_input_ids,
+            encoder_output,
             attention_mask,
             decoder_attention_mask,
             &self.embeddings,
@@ -328,7 +329,7 @@ impl PegasusForConditionalGeneration {
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *source_sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
     /// * `encoder_outputs` - Optional tuple made of a tensor of shape (*batch size*, *source_sequence_length*, *encoder_hidden_dim*) and optional vectors of tensors of length *num_encoder_layers* with shape (*batch size*, *source_sequence_length*, *hidden_size*).
     /// These correspond to the encoder last hidden state and optional hidden states/attention weights for encoder layers. When provided, the encoder hidden state will not be recalculated. Useful for generation tasks.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
     /// * `train` - boolean flag to turn on/off the dropout layers in the model. Should be set to false for inference.
     ///
@@ -436,7 +437,7 @@ impl LMHeadModel for PegasusForConditionalGeneration {
     /// # Arguments
     ///
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *sequence_length*). If None, pre-computed embeddings must be provided (see `input_embeds`)
-    /// * `layer_past` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing th elast calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
+    /// * `layer_past` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing the last calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
     /// * `attention_mask` - Optional mask of shape (*batch size*, *sequence_length*). Masked position have value 0, non-masked value 1. If None set to 1
     /// * `input_embeds` - Unused for Pegasus
     /// * `token_type_ids` - Unused for Pegasus
@@ -487,21 +488,21 @@ impl LMHeadModel for PegasusForConditionalGeneration {
     /// ```
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         cache: Cache,
-        attention_mask: &Option<Tensor>,
-        _token_type_ids: &Option<Tensor>,
-        _position_ids: &Option<Tensor>,
-        _input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        _token_type_ids: Option<&Tensor>,
+        _position_ids: Option<&Tensor>,
+        _input_embeds: Option<&Tensor>,
         encoder_outputs: Option<&Tensor>,
-        decoder_input_ids: &Option<Tensor>,
+        decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let base_model_output = match cache {
             Cache::BARTCache(cached_layer_states) => self.base_model.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
-                decoder_input_ids.as_ref().ok_or_else(|| {
+                input_ids,
+                attention_mask,
+                decoder_input_ids.ok_or_else(|| {
                     RustBertError::ValueError(
                         "Decoder input ids must be provided for Pegasus language models"
                             .to_string(),
@@ -513,9 +514,9 @@ impl LMHeadModel for PegasusForConditionalGeneration {
                 train,
             ),
             Cache::None => self.base_model.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
-                decoder_input_ids.as_ref().ok_or_else(|| {
+                input_ids,
+                attention_mask,
+                decoder_input_ids.ok_or_else(|| {
                     RustBertError::ValueError(
                         "Decoder input ids must be provided for Pegasus language models"
                             .to_string(),
@@ -690,7 +691,7 @@ impl PrivateLanguageGenerator<PegasusForConditionalGeneration, PegasusVocab, Peg
     fn get_model(&self) -> &PegasusForConditionalGeneration {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
@@ -726,6 +727,7 @@ impl PrivateLanguageGenerator<PegasusForConditionalGeneration, PegasusVocab, Peg
         scores: &mut Tensor,
         current_length: i64,
         max_length: i64,
+        _forced_bos_token_id: Option<i64>,
     ) {
         if current_length == max_length - 1 {
             self.force_token_id_generation(scores, self.get_eos_ids().as_ref().unwrap());
@@ -773,7 +775,7 @@ impl PrivateLanguageGenerator<PegasusForConditionalGeneration, PegasusVocab, Peg
     where
         S: AsRef<[&'a str]>,
     {
-        let tokens = self.get_tokenizer().encode_list(
+        let tokens = self._get_tokenizer().encode_list(
             prompt_text.as_ref(),
             max_len as usize,
             &TruncationStrategy::LongestFirst,
@@ -789,7 +791,7 @@ impl PrivateLanguageGenerator<PegasusForConditionalGeneration, PegasusVocab, Peg
         let pad_token = match pad_token_id {
             Some(value) => value,
             None => self
-                .get_tokenizer()
+                ._get_tokenizer()
                 .convert_tokens_to_ids(&[PegasusVocab::pad_value()])[0],
         };
 
