@@ -25,6 +25,8 @@ use crate::electra::ElectraConfig;
 use crate::gpt2::Gpt2Config;
 use crate::gpt_neo::GptNeoConfig;
 use crate::longformer::LongformerConfig;
+use crate::m2m_100::M2M100Config;
+use crate::mbart::MBartConfig;
 use crate::mobilebert::MobileBertConfig;
 use crate::pegasus::PegasusConfig;
 use crate::prophetnet::ProphetNetConfig;
@@ -33,20 +35,22 @@ use crate::t5::T5Config;
 use crate::xlnet::XLNetConfig;
 use crate::Config;
 use rust_tokenizers::tokenizer::{
-    AlbertTokenizer, BertTokenizer, Gpt2Tokenizer, MarianTokenizer, MultiThreadedTokenizer,
-    OpenAiGptTokenizer, PegasusTokenizer, ProphetNetTokenizer, ReformerTokenizer, RobertaTokenizer,
-    T5Tokenizer, Tokenizer, TruncationStrategy, XLMRobertaTokenizer, XLNetTokenizer,
+    AlbertTokenizer, BertTokenizer, Gpt2Tokenizer, M2M100Tokenizer, MBart50Tokenizer,
+    MarianTokenizer, MultiThreadedTokenizer, OpenAiGptTokenizer, PegasusTokenizer,
+    ProphetNetTokenizer, ReformerTokenizer, RobertaTokenizer, T5Tokenizer, Tokenizer,
+    TruncationStrategy, XLMRobertaTokenizer, XLNetTokenizer,
 };
 use rust_tokenizers::vocab::{
-    AlbertVocab, BertVocab, Gpt2Vocab, MarianVocab, OpenAiGptVocab, PegasusVocab, ProphetNetVocab,
-    ReformerVocab, RobertaVocab, T5Vocab, Vocab, XLMRobertaVocab, XLNetVocab,
+    AlbertVocab, BertVocab, Gpt2Vocab, M2M100Vocab, MBart50Vocab, MarianVocab, OpenAiGptVocab,
+    PegasusVocab, ProphetNetVocab, ReformerVocab, RobertaVocab, T5Vocab, Vocab, XLMRobertaVocab,
+    XLNetVocab,
 };
 use rust_tokenizers::{TokenIdsWithOffsets, TokenizedInput, TokensWithOffsets};
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::path::Path;
 
-#[derive(Clone, Copy, Serialize, Deserialize, Debug)]
+#[derive(Clone, Copy, Serialize, Deserialize, Debug, PartialEq)]
 /// # Identifies the type of model
 pub enum ModelType {
     Bart,
@@ -67,6 +71,8 @@ pub enum ModelType {
     Longformer,
     Pegasus,
     GPTNeo,
+    MBart,
+    M2M100,
 }
 
 /// # Abstraction that holds a model configuration, can be of any of the supported models
@@ -101,6 +107,10 @@ pub enum ConfigOption {
     Pegasus(PegasusConfig),
     /// GPT-Neo configuration
     GPTNeo(GptNeoConfig),
+    /// MBart configuration
+    MBart(MBartConfig),
+    /// M2M100 configuration
+    M2M100(M2M100Config),
 }
 
 /// # Abstraction that holds a particular tokenizer, can be of any of the supported models
@@ -129,6 +139,10 @@ pub enum TokenizerOption {
     ProphetNet(ProphetNetTokenizer),
     /// Pegasus Tokenizer
     Pegasus(PegasusTokenizer),
+    /// MBart50 Tokenizer
+    MBart50(MBart50Tokenizer),
+    /// M2M100 Tokenizer
+    M2M100(M2M100Tokenizer),
 }
 
 impl ConfigOption {
@@ -152,43 +166,64 @@ impl ConfigOption {
             ModelType::ProphetNet => ConfigOption::ProphetNet(ProphetNetConfig::from_file(path)),
             ModelType::Longformer => ConfigOption::Longformer(LongformerConfig::from_file(path)),
             ModelType::Pegasus => ConfigOption::Pegasus(PegasusConfig::from_file(path)),
+            ModelType::MBart => ConfigOption::MBart(MBartConfig::from_file(path)),
+            ModelType::M2M100 => ConfigOption::M2M100(M2M100Config::from_file(path)),
         }
     }
 
-    pub fn get_label_mapping(self) -> HashMap<i64, String> {
+    pub fn get_label_mapping(&self) -> &HashMap<i64, String> {
         match self {
             Self::Bart(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::Bert(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::DistilBert(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::Electra(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::Marian(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::MobileBert(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::Albert(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::XLNet(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::Reformer(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::ProphetNet(config) => config
                 .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::Longformer(config) => config
                 .id2label
+                .as_ref()
+                .expect("No label dictionary (id2label) provided in configuration file"),
+            Self::MBart(config) => config
+                .id2label
+                .as_ref()
+                .expect("No label dictionary (id2label) provided in configuration file"),
+            Self::M2M100(config) => config
+                .id2label
+                .as_ref()
                 .expect("No label dictionary (id2label) provided in configuration file"),
             Self::T5(_) => panic!("T5 does not use a label mapping"),
             Self::GPT2(_) => panic!("GPT2 does not use a label mapping"),
@@ -196,6 +231,28 @@ impl ConfigOption {
             Self::Pegasus(_) => panic!("Pegasus does not use a label mapping"),
         }
     }
+
+    pub fn get_max_len(&self) -> Option<i64> {
+        match self {
+            Self::Bart(config) => Some(config.max_position_embeddings),
+            Self::Bert(config) => Some(config.max_position_embeddings),
+            Self::DistilBert(config) => Some(config.max_position_embeddings),
+            Self::Electra(config) => Some(config.max_position_embeddings),
+            Self::Marian(config) => Some(config.max_position_embeddings),
+            Self::MobileBert(config) => Some(config.max_position_embeddings),
+            Self::T5(_) => None,
+            Self::Albert(config) => Some(config.max_position_embeddings),
+            Self::XLNet(_) => None,
+            Self::GPT2(config) => Some(config.n_positions),
+            Self::Reformer(config) => Some(config.max_position_embeddings),
+            Self::ProphetNet(config) => Some(config.max_position_embeddings),
+            Self::Longformer(config) => Some(config.max_position_embeddings),
+            Self::Pegasus(config) => Some(config.max_position_embeddings),
+            Self::GPTNeo(config) => Some(config.max_position_embeddings),
+            Self::MBart(config) => Some(config.max_position_embeddings),
+            Self::M2M100(config) => Some(config.max_position_embeddings),
+        }
+    }
 }
 
 impl TokenizerOption {
@@ -376,6 +433,42 @@ impl TokenizerOption {
                 }
                 TokenizerOption::Pegasus(PegasusTokenizer::from_file(vocab_path, lower_case)?)
             }
+            ModelType::MBart => {
+                if add_prefix_space.is_some() {
+                    return Err(RustBertError::InvalidConfigurationError(
+                        format!("Optional input `add_prefix_space` set to value {} but cannot be used by {:?}",
+                                add_prefix_space.unwrap(),
+                                model_type)));
+                }
+                if strip_accents.is_some() {
+                    return Err(RustBertError::InvalidConfigurationError(format!(
+                        "Optional input `strip_accents` set to value {} but cannot be used by {:?}",
+                        strip_accents.unwrap(),
+                        model_type
+                    )));
+                }
+                TokenizerOption::MBart50(MBart50Tokenizer::from_file(vocab_path, lower_case)?)
+            }
+            ModelType::M2M100 => {
+                if add_prefix_space.is_some() {
+                    return Err(RustBertError::InvalidConfigurationError(
+                        format!("Optional input `add_prefix_space` set to value {} but cannot be used by {:?}",
+                                add_prefix_space.unwrap(),
+                                model_type)));
+                }
+                if strip_accents.is_some() {
+                    return Err(RustBertError::InvalidConfigurationError(format!(
+                        "Optional input `strip_accents` set to value {} but cannot be used by {:?}",
+                        strip_accents.unwrap(),
+                        model_type
+                    )));
+                }
+                TokenizerOption::M2M100(M2M100Tokenizer::from_files(
+                    vocab_path,
+                    merges_path.expect("No merges specified!"),
+                    lower_case,
+                )?)
+            }
         };
         Ok(tokenizer)
     }
@@ -395,6 +488,8 @@ impl TokenizerOption {
             Self::Reformer(_) => ModelType::Reformer,
             Self::ProphetNet(_) => ModelType::ProphetNet,
             Self::Pegasus(_) => ModelType::Pegasus,
+            Self::MBart50(_) => ModelType::MBart,
+            Self::M2M100(_) => ModelType::M2M100,
         }
     }
 
@@ -491,6 +586,20 @@ impl TokenizerOption {
                 truncation_strategy,
                 stride,
             ),
+            Self::MBart50(ref tokenizer) => MultiThreadedTokenizer::encode_list(
+                tokenizer,
+                text_list,
+                max_len,
+                truncation_strategy,
+                stride,
+            ),
+            Self::M2M100(ref tokenizer) => MultiThreadedTokenizer::encode_list(
+                tokenizer,
+                text_list,
+                max_len,
+                truncation_strategy,
+                stride,
+            ),
         }
     }
 
@@ -587,6 +696,20 @@ impl TokenizerOption {
                 truncation_strategy,
                 stride,
             ),
+            Self::MBart50(ref tokenizer) => MultiThreadedTokenizer::encode_pair_list(
+                tokenizer,
+                text_pair_list,
+                max_len,
+                truncation_strategy,
+                stride,
+            ),
+            Self::M2M100(ref tokenizer) => MultiThreadedTokenizer::encode_pair_list(
+                tokenizer,
+                text_pair_list,
+                max_len,
+                truncation_strategy,
+                stride,
+            ),
         }
     }
 
@@ -636,6 +759,12 @@ impl TokenizerOption {
             Self::Pegasus(ref tokenizer) => {
                 tokenizer.encode(text_1, text_2, max_len, truncation_strategy, stride)
             }
+            Self::MBart50(ref tokenizer) => {
+                tokenizer.encode(text_1, text_2, max_len, truncation_strategy, stride)
+            }
+            Self::M2M100(ref tokenizer) => {
+                tokenizer.encode(text_1, text_2, max_len, truncation_strategy, stride)
+            }
         }
     }
 
@@ -654,6 +783,8 @@ impl TokenizerOption {
             Self::Reformer(ref tokenizer) => tokenizer.tokenize(text),
             Self::ProphetNet(ref tokenizer) => tokenizer.tokenize(text),
             Self::Pegasus(ref tokenizer) => tokenizer.tokenize(text),
+            Self::MBart50(ref tokenizer) => tokenizer.tokenize(text),
+            Self::M2M100(ref tokenizer) => tokenizer.tokenize(text),
         }
     }
 
@@ -672,6 +803,8 @@ impl TokenizerOption {
             Self::Reformer(ref tokenizer) => tokenizer.tokenize_with_offsets(text),
             Self::ProphetNet(ref tokenizer) => tokenizer.tokenize_with_offsets(text),
             Self::Pegasus(ref tokenizer) => tokenizer.tokenize_with_offsets(text),
+            Self::MBart50(ref tokenizer) => tokenizer.tokenize_with_offsets(text),
+            Self::M2M100(ref tokenizer) => tokenizer.tokenize_with_offsets(text),
         }
     }
 
@@ -696,6 +829,8 @@ impl TokenizerOption {
                 MultiThreadedTokenizer::tokenize_list(tokenizer, text)
             }
             Self::Pegasus(ref tokenizer) => MultiThreadedTokenizer::tokenize_list(tokenizer, text),
+            Self::MBart50(ref tokenizer) => MultiThreadedTokenizer::tokenize_list(tokenizer, text),
+            Self::M2M100(ref tokenizer) => MultiThreadedTokenizer::tokenize_list(tokenizer, text),
         }
     }
 
@@ -743,6 +878,12 @@ impl TokenizerOption {
             Self::Pegasus(ref tokenizer) => {
                 tokenizer.decode(token_ids, skip_special_tokens, clean_up_tokenization_spaces)
             }
+            Self::MBart50(ref tokenizer) => {
+                tokenizer.decode(token_ids, skip_special_tokens, clean_up_tokenization_spaces)
+            }
+            Self::M2M100(ref tokenizer) => {
+                tokenizer.decode(token_ids, skip_special_tokens, clean_up_tokenization_spaces)
+            }
         }
     }
 
@@ -801,6 +942,14 @@ impl TokenizerOption {
                 token_ids_with_offsets_1,
                 token_ids_with_offsets_2,
             ),
+            Self::MBart50(ref tokenizer) => tokenizer.build_input_with_special_tokens(
+                token_ids_with_offsets_1,
+                token_ids_with_offsets_2,
+            ),
+            Self::M2M100(ref tokenizer) => tokenizer.build_input_with_special_tokens(
+                token_ids_with_offsets_1,
+                token_ids_with_offsets_2,
+            ),
         };
         TokenizedInput {
             token_ids: token_ids_with_special_tokens.token_ids,
@@ -833,6 +982,8 @@ impl TokenizerOption {
             Self::Reformer(ref tokenizer) => tokenizer.convert_tokens_to_ids(tokens),
             Self::ProphetNet(ref tokenizer) => tokenizer.convert_tokens_to_ids(tokens),
             Self::Pegasus(ref tokenizer) => tokenizer.convert_tokens_to_ids(tokens),
+            Self::MBart50(ref tokenizer) => tokenizer.convert_tokens_to_ids(tokens),
+            Self::M2M100(ref tokenizer) => tokenizer.convert_tokens_to_ids(tokens),
         }
     }
 
@@ -887,6 +1038,14 @@ impl TokenizerOption {
                 .special_values
                 .get(PegasusVocab::unknown_value())
                 .expect("UNK token not found in vocabulary"),
+            Self::MBart50(ref tokenizer) => *MultiThreadedTokenizer::vocab(tokenizer)
+                .special_values
+                .get(MBart50Vocab::unknown_value())
+                .expect("UNK token not found in vocabulary"),
+            Self::M2M100(ref tokenizer) => *MultiThreadedTokenizer::vocab(tokenizer)
+                .special_values
+                .get(M2M100Vocab::unknown_value())
+                .expect("UNK token not found in vocabulary"),
         }
     }
 
@@ -947,6 +1106,18 @@ impl TokenizerOption {
                     .get(PegasusVocab::pad_value())
                     .expect("PAD token not found in vocabulary"),
             ),
+            Self::MBart50(ref tokenizer) => Some(
+                *MultiThreadedTokenizer::vocab(tokenizer)
+                    .special_values
+                    .get(MBart50Vocab::pad_value())
+                    .expect("PAD token not found in vocabulary"),
+            ),
+            Self::M2M100(ref tokenizer) => Some(
+                *MultiThreadedTokenizer::vocab(tokenizer)
+                    .special_values
+                    .get(M2M100Vocab::pad_value())
+                    .expect("PAD token not found in vocabulary"),
+            ),
             Self::Reformer(_) => None,
             Self::GPT2(_) => None,
             Self::OpenAiGpt(_) => None,
@@ -992,6 +1163,18 @@ impl TokenizerOption {
                     .get(ProphetNetVocab::sep_value())
                     .expect("SEP token not found in vocabulary"),
             ),
+            Self::MBart50(ref tokenizer) => Some(
+                *MultiThreadedTokenizer::vocab(tokenizer)
+                    .special_values
+                    .get(MBart50Vocab::sep_value())
+                    .expect("SEP token not found in vocabulary"),
+            ),
+            Self::M2M100(ref tokenizer) => Some(
+                *MultiThreadedTokenizer::vocab(tokenizer)
+                    .special_values
+                    .get(M2M100Vocab::sep_value())
+                    .expect("SEP token not found in vocabulary"),
+            ),
             Self::Marian(_) => None,
             Self::T5(_) => None,
             Self::GPT2(_) => None,
