@@ -353,19 +353,19 @@ impl SequenceClassificationOption {
     /// Interface method to forward_t() of the particular models.
     pub fn forward_t(
         &self,
-        input_ids: Option<Tensor>,
-        mask: Option<Tensor>,
-        token_type_ids: Option<Tensor>,
-        position_ids: Option<Tensor>,
-        input_embeds: Option<Tensor>,
+        input_ids: Option<&Tensor>,
+        mask: Option<&Tensor>,
+        token_type_ids: Option<&Tensor>,
+        position_ids: Option<&Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> Tensor {
         match *self {
             Self::Bart(ref model) => {
                 model
                     .forward_t(
-                        &input_ids.expect("`input_ids` must be provided for BART models"),
-                        mask.as_ref(),
+                        input_ids.expect("`input_ids` must be provided for BART models"),
+                        mask,
                         None,
                         None,
                         None,
@@ -393,14 +393,7 @@ impl SequenceClassificationOption {
             }
             Self::MobileBert(ref model) => {
                 model
-                    .forward_t(
-                        input_ids.as_ref(),
-                        None,
-                        None,
-                        input_embeds,
-                        mask.as_ref(),
-                        train,
-                    )
+                    .forward_t(input_ids, None, None, input_embeds, mask, train)
                     .expect("Error in mobilebert forward_t")
                     .logits
             }
@@ -431,12 +424,12 @@ impl SequenceClassificationOption {
             Self::XLNet(ref model) => {
                 model
                     .forward_t(
-                        input_ids.as_ref(),
-                        mask.as_ref(),
+                        input_ids,
+                        mask,
                         None,
                         None,
                         None,
-                        token_type_ids.as_ref(),
+                        token_type_ids,
                         input_embeds,
                         train,
                     )
@@ -444,19 +437,19 @@ impl SequenceClassificationOption {
             }
             Self::Reformer(ref model) => {
                 model
-                    .forward_t(input_ids.as_ref(), None, None, mask.as_ref(), None, train)
+                    .forward_t(input_ids, None, None, mask, None, train)
                     .expect("Error in Reformer forward pass.")
                     .logits
             }
             Self::Longformer(ref model) => {
                 model
                     .forward_t(
-                        input_ids.as_ref(),
-                        mask.as_ref(),
+                        input_ids,
+                        mask,
                         None,
-                        token_type_ids.as_ref(),
-                        position_ids.as_ref(),
-                        input_embeds.as_ref(),
+                        token_type_ids,
+                        position_ids,
+                        input_embeds,
                         train,
                     )
                     .expect("Error in Longformer forward pass.")
@@ -472,6 +465,7 @@ pub struct SequenceClassificationModel {
     sequence_classifier: SequenceClassificationOption,
     label_mapping: HashMap<i64, String>,
     var_store: VarStore,
+    max_length: usize,
 }
 
 impl SequenceClassificationModel {
@@ -514,15 +508,20 @@ impl SequenceClassificationModel {
         )?;
         let mut var_store = VarStore::new(device);
         let model_config = ConfigOption::from_file(config.model_type, config_path);
+        let max_length = model_config
+            .get_max_len()
+            .map(|v| v as usize)
+            .unwrap_or(usize::MAX);
         let sequence_classifier =
             SequenceClassificationOption::new(config.model_type, &var_store.root(), &model_config)?;
-        let label_mapping = model_config.get_label_mapping();
+        let label_mapping = model_config.get_label_mapping().clone();
         var_store.load(weights_path)?;
         Ok(SequenceClassificationModel {
             tokenizer,
             sequence_classifier,
             label_mapping,
             var_store,
+            max_length,
         })
     }
 
@@ -530,9 +529,12 @@ impl SequenceClassificationModel {
     where
         S: AsRef<[&'a str]>,
     {
-        let tokenized_input: Vec<TokenizedInput> =
-            self.tokenizer
-                .encode_list(input.as_ref(), 128, &TruncationStrategy::LongestFirst, 0);
+        let tokenized_input: Vec<TokenizedInput> = self.tokenizer.encode_list(
+            input.as_ref(),
+            self.max_length,
+            &TruncationStrategy::LongestFirst,
+            0,
+        );
         let max_len = tokenized_input
             .iter()
             .map(|input| input.token_ids.len())
@@ -588,7 +590,7 @@ impl SequenceClassificationModel {
         let input_tensor = self.prepare_for_model(input.as_ref());
         let output = no_grad(|| {
             let output = self.sequence_classifier.forward_t(
-                Some(input_tensor.copy()),
+                Some(&input_tensor),
                 None,
                 None,
                 None,
@@ -597,10 +599,10 @@ impl SequenceClassificationModel {
             );
             output.softmax(-1, Kind::Float).detach().to(Device::Cpu)
         });
-        let label_indices = output.as_ref().argmax(-1, true).squeeze1(1);
+        let label_indices = output.as_ref().argmax(-1, true).squeeze_dim(1);
         let scores = output
             .gather(1, &label_indices.unsqueeze(-1), false)
-            .squeeze1(1);
+            .squeeze_dim(1);
         let label_indices = label_indices.iter::<i64>().unwrap().collect::<Vec<i64>>();
         let scores = scores.iter::<f64>().unwrap().collect::<Vec<f64>>();
 
@@ -657,7 +659,7 @@ impl SequenceClassificationModel {
         let input_tensor = self.prepare_for_model(input.to_vec());
         let output = no_grad(|| {
             let output = self.sequence_classifier.forward_t(
-                Some(input_tensor.copy()),
+                Some(&input_tensor),
                 None,
                 None,
                 None,
