@@ -74,7 +74,7 @@ pub struct TextGenerationConfig {
     pub repetition_penalty: f64,
     /// Exponential penalty based on the length of the hypotheses generated (default: 1.0)
     pub length_penalty: f64,
-    /// Number of allowed repetitions of n-grams. Values higher than 0 turn on this feature (default: 3)
+    /// Number of allowed repetitions of n-grams. Values higher than 0 turn on this feature and will prevent repeats of n-grams with a length equal or greater to this value (default: 0)
     pub no_repeat_ngram_size: i64,
     /// Number of sequences to return for each prompt text (default: 1)
     pub num_return_sequences: i64,
@@ -141,7 +141,7 @@ impl Default for TextGenerationConfig {
             top_p: 0.9,
             repetition_penalty: 1.0,
             length_penalty: 1.0,
-            no_repeat_ngram_size: 3,
+            no_repeat_ngram_size: 0,
             num_return_sequences: 1,
             num_beam_groups: None,
             diversity_penalty: None,
@@ -229,11 +229,11 @@ impl TextGenerationOption {
     /// Interface method to access tokenizer
     pub fn get_tokenizer(&self) -> &TokenizerOption {
         match self {
-            Self::GPT(model_ref) => model_ref.get_tokenizer(),
-            Self::GPT2(model_ref) => model_ref.get_tokenizer(),
-            Self::GPTNeo(model_ref) => model_ref.get_tokenizer(),
-            Self::XLNet(model_ref) => model_ref.get_tokenizer(),
-            Self::Reformer(model_ref) => model_ref.get_tokenizer(),
+            Self::GPT(model_ref) => model_ref._get_tokenizer(),
+            Self::GPT2(model_ref) => model_ref._get_tokenizer(),
+            Self::GPTNeo(model_ref) => model_ref._get_tokenizer(),
+            Self::XLNet(model_ref) => model_ref._get_tokenizer(),
+            Self::Reformer(model_ref) => model_ref._get_tokenizer(),
         }
     }
 
@@ -249,21 +249,76 @@ impl TextGenerationOption {
         S: AsRef<[&'a str]>,
     {
         match *self {
-            Self::GPT(ref model) => {
-                model.generate_indices(prompt_texts, attention_mask, min_length, max_length, None)
-            }
-            Self::GPT2(ref model) => {
-                model.generate_indices(prompt_texts, attention_mask, min_length, max_length, None)
-            }
-            Self::GPTNeo(ref model) => {
-                model.generate_indices(prompt_texts, attention_mask, min_length, max_length, None)
-            }
-            Self::XLNet(ref model) => {
-                model.generate_indices(prompt_texts, attention_mask, min_length, max_length, None)
-            }
-            Self::Reformer(ref model) => {
-                model.generate_indices(prompt_texts, attention_mask, min_length, max_length, None)
-            }
+            Self::GPT(ref model) => model
+                .generate_indices(
+                    prompt_texts,
+                    attention_mask,
+                    min_length,
+                    max_length,
+                    None,
+                    None,
+                    None,
+                    false,
+                )
+                .into_iter()
+                .map(|output| output.indices)
+                .collect(),
+            Self::GPT2(ref model) => model
+                .generate_indices(
+                    prompt_texts,
+                    attention_mask,
+                    min_length,
+                    max_length,
+                    None,
+                    None,
+                    None,
+                    false,
+                )
+                .into_iter()
+                .map(|output| output.indices)
+                .collect(),
+            Self::GPTNeo(ref model) => model
+                .generate_indices(
+                    prompt_texts,
+                    attention_mask,
+                    min_length,
+                    max_length,
+                    None,
+                    None,
+                    None,
+                    false,
+                )
+                .into_iter()
+                .map(|output| output.indices)
+                .collect(),
+            Self::XLNet(ref model) => model
+                .generate_indices(
+                    prompt_texts,
+                    attention_mask,
+                    min_length,
+                    max_length,
+                    None,
+                    None,
+                    None,
+                    false,
+                )
+                .into_iter()
+                .map(|output| output.indices)
+                .collect(),
+            Self::Reformer(ref model) => model
+                .generate_indices(
+                    prompt_texts,
+                    attention_mask,
+                    min_length,
+                    max_length,
+                    None,
+                    None,
+                    None,
+                    false,
+                )
+                .into_iter()
+                .map(|output| output.indices)
+                .collect(),
         }
     }
 }
