@@ -123,17 +123,17 @@ use crate::mobilebert::MobileBertForTokenClassification;
 use crate::pipelines::common::{ConfigOption, ModelType, TokenizerOption};
 use crate::roberta::RobertaForTokenClassification;
 use crate::xlnet::XLNetForTokenClassification;
-use rust_tokenizers::tokenizer::{Tokenizer, TruncationStrategy};
+use rust_tokenizers::tokenizer::Tokenizer;
 use rust_tokenizers::{
-    ConsolidatableTokens, ConsolidatedTokenIterator, Mask, Offset, TokenTrait, TokenizedInput,
+    ConsolidatableTokens, ConsolidatedTokenIterator, Mask, Offset, TokenIdsWithOffsets, TokenTrait,
+    TokenizedInput,
 };
 use serde::{Deserialize, Serialize};
 use std::borrow::Borrow;
 use std::cmp::min;
 use std::collections::HashMap;
-use tch::kind::Kind::Float;
 use tch::nn::VarStore;
-use tch::{nn, no_grad, Device, Tensor};
+use tch::{nn, no_grad, Device, Kind, Tensor};
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 /// # Token generated by a `TokenClassificationModel`
@@ -178,6 +178,20 @@ impl ConsolidatableTokens<Token> for Vec<Token> {
     }
 }
 
+#[derive(Debug)]
+struct InputFeature {
+    /// Encoded input ids
+    input_ids: Vec<i64>,
+    /// Offsets reference to the original string
+    offsets: Vec<Option<Offset>>,
+    /// Token category (mask)
+    mask: Vec<Mask>,
+    /// per-token flag indicating if this feature carries the output label for this token
+    reference_feature: Vec<bool>,
+    /// Reference example index (long inputs may be broken into multiple input features)
+    example_index: usize,
+}
+
 type LabelAggregationFunction = Box<fn(&[Token]) -> (i64, String)>;
 
 /// # Enum defining the label aggregation method for sub tokens
@@ -216,6 +230,8 @@ pub struct TokenClassificationConfig {
     pub device: Device,
     /// Sub-tokens aggregation method (default: `LabelAggregationOption::First`)
     pub label_aggregation_function: LabelAggregationOption,
+    /// Batch size for predictions
+    pub batch_size: usize,
 }
 
 impl TokenClassificationConfig {
@@ -251,6 +267,7 @@ impl TokenClassificationConfig {
             add_prefix_space: add_prefix_space.into(),
             device: Device::cuda_if_available(),
             label_aggregation_function,
+            batch_size: 64,
         }
     }
 }
@@ -275,6 +292,7 @@ impl Default for TokenClassificationConfig {
             add_prefix_space: None,
             device: Device::cuda_if_available(),
             label_aggregation_function: LabelAggregationOption::First,
+            batch_size: 64,
         }
     }
 }
@@ -442,11 +460,11 @@ impl TokenClassificationOption {
 
     fn forward_t(
         &self,
-        input_ids: Option<Tensor>,
-        mask: Option<Tensor>,
-        token_type_ids: Option<Tensor>,
-        position_ids: Option<Tensor>,
-        input_embeds: Option<Tensor>,
+        input_ids: Option<&Tensor>,
+        mask: Option<&Tensor>,
+        token_type_ids: Option<&Tensor>,
+        position_ids: Option<&Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> Tensor {
         match *self {
@@ -470,14 +488,7 @@ impl TokenClassificationOption {
             }
             Self::MobileBert(ref model) => {
                 model
-                    .forward_t(
-                        input_ids.as_ref(),
-                        None,
-                        None,
-                        input_embeds,
-                        mask.as_ref(),
-                        train,
-                    )
+                    .forward_t(input_ids, None, None, input_embeds, mask, train)
                     .expect("Error in mobilebert forward_t")
                     .logits
             }
@@ -520,12 +531,12 @@ impl TokenClassificationOption {
             Self::XLNet(ref model) => {
                 model
                     .forward_t(
-                        input_ids.as_ref(),
-                        mask.as_ref(),
+                        input_ids,
+                        mask,
                         None,
                         None,
                         None,
-                        token_type_ids.as_ref(),
+                        token_type_ids,
                         input_embeds,
                         train,
                     )
@@ -534,12 +545,12 @@ impl TokenClassificationOption {
             Self::Longformer(ref model) => {
                 model
                     .forward_t(
-                        input_ids.as_ref(),
-                        mask.as_ref(),
+                        input_ids,
+                        mask,
                         None,
-                        token_type_ids.as_ref(),
-                        position_ids.as_ref(),
-                        input_embeds.as_ref(),
+                        token_type_ids,
+                        position_ids,
+                        input_embeds,
                         train,
                     )
                     .expect("Error in longformer forward_t")
@@ -556,6 +567,8 @@ pub struct TokenClassificationModel {
     label_mapping: HashMap<i64, String>,
     var_store: VarStore,
     label_aggregation_function: LabelAggregationOption,
+    max_length: usize,
+    batch_size: usize,
 }
 
 impl TokenClassificationModel {
@@ -599,9 +612,14 @@ impl TokenClassificationModel {
         )?;
         let mut var_store = VarStore::new(device);
         let model_config = ConfigOption::from_file(config.model_type, config_path);
+        let max_length = model_config
+            .get_max_len()
+            .map(|v| v as usize)
+            .unwrap_or(usize::MAX);
         let token_sequence_classifier =
             TokenClassificationOption::new(config.model_type, &var_store.root(), &model_config)?;
-        let label_mapping = model_config.get_label_mapping();
+        let label_mapping = model_config.get_label_mapping().clone();
+        let batch_size = config.batch_size;
         var_store.load(weights_path)?;
         Ok(TokenClassificationModel {
             tokenizer,
@@ -609,39 +627,126 @@ impl TokenClassificationModel {
             label_mapping,
             var_store,
             label_aggregation_function,
+            max_length,
+            batch_size,
         })
     }
 
-    fn prepare_for_model<'a, S>(&self, input: S) -> (Vec<TokenizedInput>, Tensor)
+    fn generate_features<S>(&self, input: S, example_index: usize) -> Vec<InputFeature>
     where
-        S: AsRef<[&'a str]>,
+        S: AsRef<str>,
     {
-        let tokenized_input: Vec<TokenizedInput> =
-            self.tokenizer
-                .encode_list(input.as_ref(), 128, &TruncationStrategy::LongestFirst, 0);
-        let max_len = tokenized_input
-            .iter()
-            .map(|input| input.token_ids.len())
-            .max()
-            .unwrap();
-        let tokenized_input_tensors: Vec<tch::Tensor> = tokenized_input
-            .iter()
-            .map(|input| input.token_ids.clone())
-            .map(|mut input| {
-                input.extend(vec![
-                    self.tokenizer.get_pad_id().expect(
-                        "The Tokenizer used for token classification should contain a PAD id"
-                    );
-                    max_len - input.len()
-                ]);
-                input
-            })
-            .map(|input| Tensor::of_slice(&(input)))
-            .collect::<Vec<_>>();
-        (
-            tokenized_input,
-            Tensor::stack(tokenized_input_tensors.as_slice(), 0).to(self.var_store.device()),
-        )
+        let tokenized_input = self.tokenizer.tokenize_with_offsets(input.as_ref());
+        let encoded_input = TokenIdsWithOffsets {
+            ids: self
+                .tokenizer
+                .convert_tokens_to_ids(&tokenized_input.tokens),
+            offsets: tokenized_input.offsets,
+            reference_offsets: tokenized_input.reference_offsets,
+            masks: tokenized_input.masks,
+        };
+
+        let sequence_added_tokens = self
+            .tokenizer
+            .build_input_with_special_tokens(
+                TokenIdsWithOffsets {
+                    ids: vec![],
+                    offsets: vec![],
+                    reference_offsets: vec![],
+                    masks: vec![],
+                },
+                None,
+            )
+            .token_ids
+            .len();
+
+        let max_content_length = self.max_length - sequence_added_tokens;
+        let doc_stride = self.max_length / 4;
+
+        let mut spans: Vec<InputFeature> = vec![];
+        let mut start_token = 0_usize;
+        let total_length = encoded_input.ids.len();
+
+        while (spans.len() * doc_stride as usize) < encoded_input.ids.len() {
+            let end_token = min(start_token + max_content_length, total_length);
+            let sub_encoded_input = TokenIdsWithOffsets {
+                ids: encoded_input.ids[start_token..end_token].to_vec(),
+                offsets: encoded_input.offsets[start_token..end_token].to_vec(),
+                reference_offsets: encoded_input.reference_offsets[start_token..end_token].to_vec(),
+                masks: encoded_input.masks[start_token..end_token].to_vec(),
+            };
+
+            let encoded_span = self
+                .tokenizer
+                .build_input_with_special_tokens(sub_encoded_input, None);
+
+            let reference_feature = self.get_reference_feature_flag(
+                start_token,
+                end_token,
+                total_length,
+                doc_stride,
+                &encoded_span,
+            );
+
+            let feature = InputFeature {
+                input_ids: encoded_span.token_ids,
+                offsets: encoded_span.token_offsets,
+                mask: encoded_span.mask,
+                reference_feature,
+                example_index,
+            };
+            spans.push(feature);
+            if end_token == encoded_input.ids.len() {
+                break;
+            }
+            start_token = end_token - doc_stride;
+        }
+        spans
+    }
+
+    fn get_reference_feature_flag(
+        &self,
+        start_token: usize,
+        end_token: usize,
+        total_length: usize,
+        doc_stride: usize,
+        encoded_span: &TokenizedInput,
+    ) -> Vec<bool> {
+        // set halfway through the doc_stride to be false if the feature is not the first/last
+        let start_cutoff = if start_token > 0 {
+            let leading_special_tokens = {
+                let mut counter = 0;
+                let mut masks = encoded_span.mask.iter();
+                while masks.next().unwrap_or(&Mask::None) == &Mask::Special {
+                    counter += 1;
+                }
+                counter
+            };
+            doc_stride / 2 + leading_special_tokens
+        } else {
+            0
+        };
+        let end_cutoff = if end_token < total_length {
+            let trailing_special_tokens = {
+                let mut counter = 0;
+                let mut masks = encoded_span.mask.iter().rev();
+                while masks.next().unwrap_or(&Mask::None) == &Mask::Special {
+                    counter += 1;
+                }
+                counter
+            };
+            encoded_span.token_ids.len() - doc_stride / 2 - trailing_special_tokens
+        } else {
+            encoded_span.token_ids.len()
+        };
+        let mut reference_feature = vec![true; encoded_span.token_ids.len()];
+        reference_feature[..start_cutoff]
+            .iter_mut()
+            .for_each(|v| *v = false);
+        reference_feature[end_cutoff..]
+            .iter_mut()
+            .for_each(|v| *v = false);
+        reference_feature
     }
 
     /// Classify tokens in a text sequence
@@ -680,63 +785,141 @@ impl TokenClassificationModel {
     where
         S: AsRef<[&'a str]>,
     {
-        let (tokenized_input, input_tensor) = self.prepare_for_model(input.as_ref());
-        let output = no_grad(|| {
-            self.token_sequence_classifier.forward_t(
-                Some(input_tensor.copy()),
-                None,
-                None,
-                None,
-                None,
-                false,
-            )
-        });
-        let output = output.detach().to(Device::Cpu);
-        let score: Tensor = output.exp() / output.exp().sum1(&[-1], true, Float);
-        let labels_idx = &score.argmax(-1, true);
-        let mut tokens: Vec<Vec<Token>> = vec![];
-        for sentence_idx in 0..labels_idx.size()[0] {
-            let mut sequence_tokens = vec![];
-            let labels = labels_idx.get(sentence_idx);
-            let sentence_tokens = &tokenized_input[sentence_idx as usize];
-            let original_chars = input.as_ref()[sentence_idx as usize]
-                .chars()
-                .collect::<Vec<char>>();
-            let mut word_idx: u16 = 0;
-            for position_idx in 0..sentence_tokens.token_ids.len() {
-                let mask = sentence_tokens.mask[position_idx];
-                if (mask == Mask::Special) & (!return_special) {
-                    continue;
-                }
-                if !(mask == Mask::Continuation) {
-                    word_idx += 1;
-                }
-                let token = {
-                    self.decode_token(
-                        &original_chars,
-                        sentence_tokens,
-                        &input_tensor,
-                        &labels,
-                        &score,
-                        sentence_idx,
-                        position_idx as i64,
-                        word_idx - 1,
-                    )
-                };
-                sequence_tokens.push(token);
-            }
-            tokens.push(sequence_tokens);
+        let mut features: Vec<InputFeature> = input
+            .as_ref()
+            .iter()
+            .enumerate()
+            .map(|(example_index, example)| self.generate_features(example, example_index))
+            .flatten()
+            .collect();
+
+        let mut example_tokens_map: HashMap<usize, Vec<Token>> = HashMap::new();
+        for example_idx in 0..input.as_ref().len() {
+            example_tokens_map.insert(example_idx, Vec::new());
         }
+        let mut start = 0usize;
+        let len_features = features.len();
+
+        while start < len_features {
+            let end = start + min(len_features - start, self.batch_size);
+
+            no_grad(|| {
+                let batch_features = &mut features[start..end];
+                let (input_ids, attention_masks) = self.pad_features(batch_features);
+                let output = self.token_sequence_classifier.forward_t(
+                    Some(&input_ids),
+                    Some(&attention_masks),
+                    None,
+                    None,
+                    None,
+                    false,
+                );
+                let score = output.exp() / output.exp().sum_dim_intlist(&[-1], true, Kind::Float);
+                let label_indices = score.argmax(-1, true);
+                for sentence_idx in 0..label_indices.size()[0] {
+                    let labels = label_indices.get(sentence_idx);
+                    let feature = &features[sentence_idx as usize];
+                    let sentence_reference_flag = &feature.reference_feature;
+                    let original_chars = input.as_ref()[feature.example_index]
+                        .chars()
+                        .collect::<Vec<char>>();
+                    let mut word_idx: u16 = 0;
+                    for position_idx in sentence_reference_flag
+                        .iter()
+                        .enumerate()
+                        .filter(|(_, flag)| **flag)
+                        .map(|(pos, _)| pos)
+                    {
+                        let mask = feature.mask[position_idx];
+                        if (mask == Mask::Special) & (!return_special) {
+                            continue;
+                        }
+                        if !(mask == Mask::Continuation) {
+                            word_idx += 1;
+                        }
+                        let token = {
+                            self.decode_token(
+                                &original_chars,
+                                feature,
+                                &input_ids,
+                                &labels,
+                                &score,
+                                sentence_idx,
+                                position_idx as i64,
+                                word_idx,
+                            )
+                        };
+                        example_tokens_map
+                            .get_mut(&(feature.example_index))
+                            .unwrap()
+                            .push(token);
+                    }
+                }
+            });
+            start = end;
+        }
+        let mut tokens = example_tokens_map
+            .into_iter()
+            .collect::<Vec<(usize, Vec<Token>)>>();
+        tokens.sort_by_key(|kv| kv.0);
+        let mut tokens = tokens
+            .into_iter()
+            .map(|(_, v)| v)
+            .collect::<Vec<Vec<Token>>>();
+
         if consolidate_sub_tokens {
             self.consolidate_tokens(&mut tokens, &self.label_aggregation_function);
         }
         tokens
     }
 
+    fn pad_features(&self, features: &mut [InputFeature]) -> (Tensor, Tensor) {
+        let max_len = features
+            .iter()
+            .map(|feature| feature.input_ids.len())
+            .max()
+            .unwrap();
+
+        let attention_masks = features
+            .iter()
+            .map(|feature| &feature.input_ids)
+            .map(|input| {
+                let mut attention_mask = vec![1; input.len()];
+                attention_mask.append(&mut vec![0; max_len - attention_mask.len()]);
+                attention_mask
+            })
+            .map(|input| Tensor::of_slice(&(input)))
+            .collect::<Vec<_>>();
+
+        for feature in features.iter_mut() {
+            feature
+                .offsets
+                .append(&mut vec![None; max_len - feature.input_ids.len()]);
+            feature.input_ids.append(&mut vec![
+                self.tokenizer.get_pad_id().expect(
+                    "Only tokenizers with a padding index can be used for token classification"
+                );
+                max_len - feature.input_ids.len()
+            ]);
+            feature
+                .reference_feature
+                .append(&mut vec![false; max_len - feature.input_ids.len()]);
+        }
+
+        let padded_input_ids = features
+            .iter()
+            .map(|input| Tensor::of_slice(input.input_ids.as_slice()))
+            .collect::<Vec<_>>();
+
+        let input_ids = Tensor::stack(&padded_input_ids, 0).to(self.var_store.device());
+        let attention_masks = Tensor::stack(&attention_masks, 0).to(self.var_store.device());
+        (input_ids, attention_masks)
+    }
+
     fn decode_token(
         &self,
         original_sentence_chars: &[char],
-        sentence_tokens: &TokenizedInput,
+        sentence_tokens: &InputFeature,
         input_tensor: &Tensor,
         labels: &Tensor,
         score: &Tensor,
@@ -747,7 +930,7 @@ impl TokenClassificationModel {
         let label_id = labels.int64_value(&[position_idx as i64]);
         let token_id = input_tensor.int64_value(&[sentence_idx, position_idx as i64]);
 
-        let offsets = &sentence_tokens.token_offsets[position_idx as usize];
+        let offsets = &sentence_tokens.offsets[position_idx as usize];
 
         let text = match offsets {
             None => match self.tokenizer {
