@@ -371,19 +371,19 @@ impl ZeroShotClassificationOption {
     /// Interface method to forward_t() of the particular models.
     pub fn forward_t(
         &self,
-        input_ids: Option<Tensor>,
-        mask: Option<Tensor>,
-        token_type_ids: Option<Tensor>,
-        position_ids: Option<Tensor>,
-        input_embeds: Option<Tensor>,
+        input_ids: Option<&Tensor>,
+        mask: Option<&Tensor>,
+        token_type_ids: Option<&Tensor>,
+        position_ids: Option<&Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> Tensor {
         match *self {
             Self::Bart(ref model) => {
                 model
                     .forward_t(
-                        &input_ids.expect("`input_ids` must be provided for BART models"),
-                        mask.as_ref(),
+                        input_ids.expect("`input_ids` must be provided for BART models"),
+                        mask,
                         None,
                         None,
                         None,
@@ -411,14 +411,7 @@ impl ZeroShotClassificationOption {
             }
             Self::MobileBert(ref model) => {
                 model
-                    .forward_t(
-                        input_ids.as_ref(),
-                        None,
-                        None,
-                        input_embeds,
-                        mask.as_ref(),
-                        train,
-                    )
+                    .forward_t(input_ids, None, None, input_embeds, mask, train)
                     .expect("Error in mobilebert forward_t")
                     .logits
             }
@@ -449,12 +442,12 @@ impl ZeroShotClassificationOption {
             Self::XLNet(ref model) => {
                 model
                     .forward_t(
-                        input_ids.as_ref(),
-                        mask.as_ref(),
+                        input_ids,
+                        mask,
                         None,
                         None,
                         None,
-                        token_type_ids.as_ref(),
+                        token_type_ids,
                         input_embeds,
                         train,
                     )
@@ -463,12 +456,12 @@ impl ZeroShotClassificationOption {
             Self::Longformer(ref model) => {
                 model
                     .forward_t(
-                        input_ids.as_ref(),
-                        mask.as_ref(),
+                        input_ids,
+                        mask,
                         None,
-                        token_type_ids.as_ref(),
-                        position_ids.as_ref(),
-                        input_embeds.as_ref(),
+                        token_type_ids,
+                        position_ids,
+                        input_embeds,
                         train,
                     )
                     .expect("Error in Longformer forward pass.")
@@ -676,8 +669,8 @@ impl ZeroShotClassificationModel {
             self.prepare_for_model(inputs.as_ref(), labels.as_ref(), template, max_length);
         let output = no_grad(|| {
             let output = self.zero_shot_classifier.forward_t(
-                Some(input_tensor),
-                Some(mask),
+                Some(&input_tensor),
+                Some(&mask),
                 None,
                 None,
                 None,
@@ -687,10 +680,10 @@ impl ZeroShotClassificationModel {
         });
 
         let scores = output.softmax(1, Float).select(-1, -1);
-        let label_indices = scores.as_ref().argmax(-1, true).squeeze1(1);
+        let label_indices = scores.as_ref().argmax(-1, true).squeeze_dim(1);
         let scores = scores
             .gather(1, &label_indices.unsqueeze(-1), false)
-            .squeeze1(1);
+            .squeeze_dim(1);
         let label_indices = label_indices.iter::<i64>().unwrap().collect::<Vec<i64>>();
         let scores = scores.iter::<f64>().unwrap().collect::<Vec<f64>>();
 
@@ -817,8 +810,8 @@ impl ZeroShotClassificationModel {
             self.prepare_for_model(inputs.as_ref(), labels.as_ref(), template, max_length);
         let output = no_grad(|| {
             let output = self.zero_shot_classifier.forward_t(
-                Some(input_tensor),
-                Some(mask),
+                Some(&input_tensor),
+                Some(&mask),
                 None,
                 None,
                 None,
