@@ -11,6 +11,7 @@
 // limitations under the License.
 
 use crate::common::dropout::Dropout;
+use crate::common::embeddings::process_ids_embeddings_pair;
 use crate::prophetnet::attention::{
     compute_all_stream_relative_buckets, LayerState, ProphetNetAttention, ProphetNetFeedForward,
     ProphetNetNgramAttention,
@@ -251,28 +252,15 @@ impl ProphetNetDecoder {
         word_embeddings: Option<&nn::Embedding>,
         train: bool,
     ) -> Result<ProphetNetDecoderOutput, RustBertError> {
-        let calc_input_embeddings = if let Some(input_ids) = input_ids {
-            if input_embeds.is_none() {
-                Some(input_ids.apply(match word_embeddings {
-                    Some(value) => value,
-                    None => {
-                        return Err(RustBertError::ValueError(
-                            "Embeddings must be provided if input_embeds is not given".into(),
-                        ));
-                    }
-                }))
-            } else {
-                return Err(RustBertError::ValueError(
-                    "Only one of input ids or input embeddings may be set".into(),
-                ));
-            }
-        } else if input_embeds.is_some() {
-            None
-        } else {
-            return Err(RustBertError::ValueError(
-                "At least one of input ids or input embeddings must be set".into(),
-            ));
-        };
+        let (calc_input_embeddings, _, _) = process_ids_embeddings_pair(
+            input_ids,
+            input_embeds,
+            word_embeddings.ok_or_else(|| {
+                RustBertError::ValueError(
+                    "Embeddings must be provided if input_embeds is not given".into(),
+                )
+            })?,
+        )?;
         let input_embeds = input_embeds.unwrap_or_else(|| calc_input_embeddings.as_ref().unwrap());
 
         let input_size = input_embeds.size();
@@ -308,9 +296,9 @@ impl ProphetNetDecoder {
 
         let hidden_states = (input_embeds + main_stream_pos_embed).transpose(0, 1);
 
-        let (mut ngram_hidden_states, extended_attention_mask, extended_predict_attention_mask) =
+        let (mut ngram_hidden_states, extended_attention_mask, extended_predict_attention_mask) = {
+            let mut ngram_hidden_states = Vec::with_capacity(self.ngram as usize);
             if old_layer_states.is_some() {
-                let mut ngram_hidden_states = Vec::with_capacity(self.ngram as usize);
                 for ngram in 0..self.ngram {
                     ngram_hidden_states.push(
                         (&self.ngram_embeddings.get(ngram - 1) + &predicting_stream_pos_embed)
@@ -320,7 +308,6 @@ impl ProphetNetDecoder {
                 }
                 (ngram_hidden_states, None, None)
             } else {
-                let mut ngram_hidden_states = Vec::with_capacity(self.ngram as usize);
                 for ngram in 0..self.ngram {
                     ngram_hidden_states.push(
                         (&self.ngram_embeddings.get(ngram - 1) + &predicting_stream_pos_embed)
@@ -336,7 +323,8 @@ impl ProphetNetDecoder {
                     Some(extended_attention_mask),
                     Some(extended_predict_attention_mask),
                 )
-            };
+            }
+        };
 
         let extended_encoder_attention_mask =
             encoder_attention_mask.map(|encoder_attention_mask_value| {
@@ -399,20 +387,6 @@ impl ProphetNetDecoder {
                 (None, None)
             };
             let temp = if let Some(x_value) = &x {
-                if let Some(main_stream_hidden_states) = all_main_stream_hidden_states.borrow_mut()
-                {
-                    main_stream_hidden_states
-                        .push(x_value.slice(0, 0, sequence_length, 1).transpose(0, 1));
-                }
-                if let Some(ngram_stream_hidden_states) =
-                    all_ngram_stream_hidden_states.borrow_mut()
-                {
-                    ngram_stream_hidden_states.push(
-                        x_value
-                            .slice(0, sequence_length, x_value.size()[0], 1)
-                            .transpose(0, 1),
-                    );
-                }
                 layer.forward_t(
                     x_value,
                     encoder_hidden_states.as_ref(),
@@ -426,23 +400,6 @@ impl ProphetNetDecoder {
                     train,
                 )
             } else {
-                if let Some(main_stream_hidden_states) = all_main_stream_hidden_states.borrow_mut()
-                {
-                    main_stream_hidden_states.push(
-                        hidden_states
-                            .slice(0, 0, sequence_length, 1)
-                            .transpose(0, 1),
-                    );
-                }
-                if let Some(ngram_stream_hidden_states) =
-                    all_ngram_stream_hidden_states.borrow_mut()
-                {
-                    ngram_stream_hidden_states.push(
-                        hidden_states
-                            .slice(0, sequence_length, hidden_states.size()[0], 1)
-                            .transpose(0, 1),
-                    );
-                }
                 layer.forward_t(
                     &hidden_states,
                     encoder_hidden_states.as_ref(),
@@ -467,24 +424,24 @@ impl ProphetNetDecoder {
             if let Some(all_cross_attentions) = all_cross_attentions.borrow_mut() {
                 all_cross_attentions.push(temp.cross_attention_weights.unwrap());
             };
+            if let Some(main_stream_hidden_states) = all_main_stream_hidden_states.borrow_mut() {
+                main_stream_hidden_states.push(
+                    x.as_ref()
+                        .unwrap()
+                        .slice(0, 0, sequence_length, 1)
+                        .transpose(0, 1),
+                );
+            };
+            if let Some(ngram_stream_hidden_states) = all_ngram_stream_hidden_states.borrow_mut() {
+                ngram_stream_hidden_states.push(
+                    x.as_ref()
+                        .unwrap()
+                        .slice(0, sequence_length, x.as_ref().unwrap().size()[0], 1)
+                        .transpose(0, 1),
+                );
+            };
             next_decoder_cache.push(temp.layer_states);
         }
-        if let Some(main_stream_hidden_states) = all_main_stream_hidden_states.borrow_mut() {
-            main_stream_hidden_states.push(
-                x.as_ref()
-                    .unwrap()
-                    .slice(0, 0, sequence_length, 1)
-                    .transpose(0, 1),
-            );
-        }
-        if let Some(ngram_stream_hidden_states) = all_ngram_stream_hidden_states.borrow_mut() {
-            ngram_stream_hidden_states.push(
-                x.as_ref()
-                    .unwrap()
-                    .slice(0, sequence_length, x.as_ref().unwrap().size()[0], 1)
-                    .transpose(0, 1),
-            );
-        }
         let x = x.unwrap();
 
         let last_hidden_state = x.slice(0, 0, sequence_length, 1).transpose(0, 1);
@@ -510,7 +467,7 @@ impl ProphetNetDecoder {
         let input_size = position_ids.size();
         let (batch_size, sequence_length) = (input_size[0], input_size[1]);
 
-        let position_ids = Tensor::arange1(
+        let position_ids = Tensor::arange_start(
             1,
             self.max_target_positions,
             (Kind::Int64, position_ids.device()),
