@@ -42,12 +42,12 @@ pub struct ProphetNetConfigResources;
 pub struct ProphetNetVocabResources;
 
 impl ProphetNetModelResources {
-    /// Shared under MIT license by the Microsoft team at https://github.com/microsoft/ProphetNet. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://github.com/microsoft/ProphetNet>. Modified with conversion to C-array format.
     pub const PROPHETNET_LARGE_UNCASED: (&'static str, &'static str) = (
         "prophetnet-large-uncased/model",
         "https://huggingface.co/microsoft/prophetnet-large-uncased/resolve/main/rust_model.ot",
     );
-    /// Shared under MIT license by the Microsoft team at https://github.com/microsoft/ProphetNet. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://github.com/microsoft/ProphetNet>. Modified with conversion to C-array format.
     pub const PROPHETNET_LARGE_CNN_DM: (&'static str, &'static str) = (
         "prophetnet-large-uncased-cnndm/model",
         "https://huggingface.co/microsoft/prophetnet-large-uncased-cnndm/resolve/main/rust_model.ot",
@@ -55,12 +55,12 @@ impl ProphetNetModelResources {
 }
 
 impl ProphetNetConfigResources {
-    /// Shared under MIT license by the Microsoft team at https://github.com/microsoft/ProphetNet. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://github.com/microsoft/ProphetNet>. Modified with conversion to C-array format.
     pub const PROPHETNET_LARGE_UNCASED: (&'static str, &'static str) = (
         "prophetnet-large-uncased/config",
         "https://huggingface.co/microsoft/prophetnet-large-uncased/resolve/main/config.json",
     );
-    /// Shared under MIT license by the Microsoft team at https://github.com/microsoft/ProphetNet. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://github.com/microsoft/ProphetNet>. Modified with conversion to C-array format.
     pub const PROPHETNET_LARGE_CNN_DM: (&'static str, &'static str) = (
         "prophetnet-large-uncased-cnndm/config",
         "https://huggingface.co/microsoft/prophetnet-large-uncased-cnndm/resolve/main/config.json",
@@ -68,12 +68,12 @@ impl ProphetNetConfigResources {
 }
 
 impl ProphetNetVocabResources {
-    /// Shared under MIT license by the Microsoft team at https://github.com/microsoft/ProphetNet. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://github.com/microsoft/ProphetNet>. Modified with conversion to C-array format.
     pub const PROPHETNET_LARGE_UNCASED: (&'static str, &'static str) = (
         "prophetnet-large-uncased/vocab",
         "https://huggingface.co/microsoft/prophetnet-large-uncased/resolve/main/prophetnet.tokenizer",
     );
-    /// Shared under MIT license by the Microsoft team at https://github.com/microsoft/ProphetNet. Modified with conversion to C-array format.
+    /// Shared under MIT license by the Microsoft team at <https://github.com/microsoft/ProphetNet>. Modified with conversion to C-array format.
     pub const PROPHETNET_LARGE_CNN_DM: (&'static str, &'static str) = (
         "prophetnet-large-uncased-cnndm/vocab",
         "https://huggingface.co/microsoft/prophetnet-large-uncased-cnndm/resolve/main/prophetnet.tokenizer",
@@ -120,7 +120,7 @@ pub struct ProphetNetConfig {
     pub add_cross_attention: Option<bool>,
 }
 
-impl Config<ProphetNetConfig> for ProphetNetConfig {}
+impl Config for ProphetNetConfig {}
 
 /// # ProphetNet Base model
 /// Base architecture for ProphetNet models. Task-specific models will be built from this common base model
@@ -190,7 +190,7 @@ impl ProphetNetModel {
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *sequence_length*). This or `input_embeds` must be provided.
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
     /// * `input_embeds` - Optional input tensor of shape (*batch size*, *sequence_length*, *embeddings dimension*). This or `input_ids` must be provided.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
     /// * `encoder_hidden_states` - Optional tensor of shape (*batch size*, *source_sequence_length*, *encoder_hidden_dim*) corresponding to pre-calculated encoder hidden states (useful for conditional generation)
     /// These correspond to the encoder last hidden state and optional hidden states/attention weights for encoder layers. When provided, the encoder hidden state will not be recalculated. Useful for generation tasks.
@@ -393,7 +393,7 @@ impl ProphetNetForConditionalGeneration {
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *sequence_length*). This or `input_embeds` must be provided.
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
     /// * `input_embeds` - Optional input tensor of shape (*batch size*, *sequence_length*, *embeddings dimension*). This or `input_ids` must be provided.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
     /// * `encoder_hidden_states` - Optional tensor of shape (*batch size*, *source_sequence_length*, *encoder_hidden_dim*) corresponding to pre-calculated encoder hidden states (useful for conditional generation)
     /// These correspond to the encoder last hidden state and optional hidden states/attention weights for encoder layers. When provided, the encoder hidden state will not be recalculated. Useful for generation tasks.
@@ -557,22 +557,22 @@ impl ProphetNetForConditionalGeneration {
 impl LMHeadModel for ProphetNetForConditionalGeneration {
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         cache: Cache,
-        attention_mask: &Option<Tensor>,
-        _token_type_ids: &Option<Tensor>,
-        _position_ids: &Option<Tensor>,
-        input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        _token_type_ids: Option<&Tensor>,
+        _position_ids: Option<&Tensor>,
+        input_embeds: Option<&Tensor>,
         encoder_outputs: Option<&Tensor>,
-        decoder_input_ids: &Option<Tensor>,
+        decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let base_model_output = match cache {
             Cache::ProphetNetCache(cached_layer_states) => self.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
-                input_embeds.as_ref(),
-                decoder_input_ids.as_ref(),
+                input_ids,
+                attention_mask,
+                input_embeds,
+                decoder_input_ids,
                 None,
                 encoder_outputs,
                 cached_layer_states,
@@ -580,10 +580,10 @@ impl LMHeadModel for ProphetNetForConditionalGeneration {
                 train,
             )?,
             Cache::None => self.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
-                input_embeds.as_ref(),
-                decoder_input_ids.as_ref(),
+                input_ids,
+                attention_mask,
+                input_embeds,
+                decoder_input_ids,
                 None,
                 encoder_outputs,
                 None,
@@ -693,7 +693,7 @@ impl ProphetNetForCausalGeneration {
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *sequence_length*). This or `input_embeds` must be provided.
     /// * `attention_mask` - Optional attention mask of shape (*batch size*, *sequence_length*) for the encoder positions. Positions with a mask with value 0 will be masked.
     /// * `input_embeds` - Optional input tensor of shape (*batch size*, *sequence_length*, *embeddings dimension*). This or `input_ids` must be provided.
-    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialiazed with a BOS token)
+    /// * `decoder_input_ids` - Optional input tensor of shape (*batch size*, *target_sequence_length*). Must be provided when running in generation mode (e.g. initialized with a BOS token)
     /// * `old_layer_states` - Optional Vector `Option<Vec<Option<&LayerState>, Option<&LayerState>>>` of length *n_layer* containing tuples with the past keys and values for both the self attention and the encoder cross attention of each layer of the decoder.
     /// * `decoder_input_embeds` - Optional input tensor of shape (*batch size*, *target_sequence_length*, *embeddings dimension*). This or `decoder_input_ids` must be provided.
     /// * `train` - boolean flag to turn on/off the dropout layers in the model. Should be set to false for inference.
@@ -993,7 +993,7 @@ impl
     fn get_model(&self) -> &ProphetNetForConditionalGeneration {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
@@ -1069,7 +1069,7 @@ impl
     where
         S: AsRef<[&'a str]>,
     {
-        let tokens = self.get_tokenizer().encode_list(
+        let tokens = self._get_tokenizer().encode_list(
             prompt_text.as_ref(),
             max_len as usize,
             &TruncationStrategy::LongestFirst,
@@ -1085,7 +1085,7 @@ impl
         let pad_token = match pad_token_id {
             Some(value) => value,
             None => self
-                .get_tokenizer()
+                ._get_tokenizer()
                 .convert_tokens_to_ids(&[ProphetNetVocab::unknown_value()])[0],
         };
 
