@@ -22,6 +22,7 @@ use tch::{nn, Device, Kind, Tensor};
 
 use crate::common::activations::Activation;
 use crate::common::dropout::Dropout;
+use crate::common::embeddings::get_shape_and_device_from_ids_embeddings_pair;
 use crate::common::resources::{RemoteResource, Resource};
 use crate::gpt2::{Gpt2ConfigResources, Gpt2ModelResources, Gpt2VocabResources};
 use crate::pipelines::common::{ModelType, TokenizerOption};
@@ -47,7 +48,7 @@ pub struct ReformerConfigResources;
 pub struct ReformerVocabResources;
 
 impl ReformerModelResources {
-    /// Shared under Apache 2.0 license by the Trax Authors at https://github.com/google/trax/tree/master/trax/models/reformer. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Trax Authors at <https://github.com/google/trax/tree/master/trax/models/reformer>. Modified with conversion to C-array format.
     pub const CRIME_AND_PUNISHMENT: (&'static str, &'static str) = (
         "reformer-crime-punishment/model",
         "https://huggingface.co/google/reformer-crime-and-punishment/resolve/main/rust_model.ot",
@@ -55,7 +56,7 @@ impl ReformerModelResources {
 }
 
 impl ReformerConfigResources {
-    /// Shared under Apache 2.0 license by the Trax Authors at https://github.com/google/trax/tree/master/trax/models/reformer. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Trax Authors at <https://github.com/google/trax/tree/master/trax/models/reformer>. Modified with conversion to C-array format.
     pub const CRIME_AND_PUNISHMENT: (&'static str, &'static str) = (
         "reformer-crime-punishment/config",
         "https://huggingface.co/google/reformer-crime-and-punishment/resolve/main/config.json",
@@ -63,7 +64,7 @@ impl ReformerConfigResources {
 }
 
 impl ReformerVocabResources {
-    /// Shared under Apache 2.0 license by the Trax Authors at https://github.com/google/trax/tree/master/trax/models/reformer. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the Trax Authors at <https://github.com/google/trax/tree/master/trax/models/reformer>. Modified with conversion to C-array format.
     pub const CRIME_AND_PUNISHMENT: (&'static str, &'static str) = (
         "reformer-crime-punishment/spiece",
         "https://huggingface.co/google/reformer-crime-and-punishment/resolve/main/spiece.model",
@@ -115,7 +116,7 @@ pub struct ReformerConfig {
     pub output_hidden_states: Option<bool>,
 }
 
-impl Config<ReformerConfig> for ReformerConfig {}
+impl Config for ReformerConfig {}
 
 pub struct ReformerLMHead {
     decoder: nn::Linear,
@@ -293,21 +294,14 @@ impl ReformerModel {
         &self,
         input_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         num_hashes: Option<i64>,
         old_layer_states: Option<Vec<Option<LayerState>>>,
         train: bool,
     ) -> Result<ReformerModelOutput, RustBertError> {
-        let (input_shape, device) = if let Some(input_ids) = input_ids {
-            (input_ids.size(), input_ids.device())
-        } else if let Some(input_embeds) = &input_embeds {
-            (input_embeds.size(), input_embeds.device())
-        } else {
-            return Err(RustBertError::ValueError(
-                "At least one of input ids or input embeddings must be set".into(),
-            ));
-        };
+        let (input_shape, device) =
+            get_shape_and_device_from_ids_embeddings_pair(input_ids, input_embeds)?;
 
         let original_sequence_length = *input_shape.last().unwrap();
 
@@ -341,7 +335,7 @@ impl ReformerModel {
             let embedding_output = self.embeddings.forward_t(
                 padded_input.input_ids.as_ref(),
                 padded_input.position_ids.as_ref(),
-                padded_input.input_embeds,
+                padded_input.input_embeds.as_ref(),
                 start_idx_pos_encodings,
                 train,
             )?;
@@ -383,7 +377,7 @@ impl ReformerModel {
     fn pad_to_mult_of_chunk_length(
         &self,
         input_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         position_ids: Option<&Tensor>,
         input_shape: &[i64],
@@ -418,10 +412,9 @@ impl ReformerModel {
             let input_ids = Tensor::cat(&[input_ids, &input_ids_padding], -1);
             new_input_shape = input_ids.size();
             let position_ids = if let Some(position_ids) = position_ids {
-                let position_ids_padding = Tensor::arange2(
+                let position_ids_padding = Tensor::arange_start(
                     *input_shape.last().unwrap(),
                     self.least_common_mult_chunk_length,
-                    1,
                     (Kind::Int64, device),
                 )
                 .unsqueeze(0)
@@ -443,7 +436,7 @@ impl ReformerModel {
                 *input_shape.last().unwrap(),
                 false,
             )?;
-            let input_embeds = Tensor::cat(&[input_embeds, input_embeds_padding], -1);
+            let input_embeds = Tensor::cat(&[input_embeds, &input_embeds_padding], -1);
             new_input_shape = input_embeds.size();
             Some(input_embeds)
         } else {
@@ -568,7 +561,7 @@ impl ReformerModelWithLMHead {
     ///
     /// let model_output = no_grad(|| {
     ///     reformer_model.forward_t(
-    ///         Some(&input_tensor),    
+    ///         Some(&input_tensor),
     ///         Some(&input_positions),
     ///         None,
     ///         Some(&attention_mask),
@@ -582,7 +575,7 @@ impl ReformerModelWithLMHead {
         &self,
         input_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         num_hashes: Option<i64>,
         old_layer_states: Option<Vec<Option<LayerState>>>,
@@ -612,35 +605,27 @@ impl ReformerModelWithLMHead {
 impl LMHeadModel for ReformerModelWithLMHead {
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         cache: Cache,
-        attention_mask: &Option<Tensor>,
-        _token_type_ids: &Option<Tensor>,
-        _position_ids: &Option<Tensor>,
-        _input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        _token_type_ids: Option<&Tensor>,
+        _position_ids: Option<&Tensor>,
+        _input_embeds: Option<&Tensor>,
         _encoder_outputs: Option<&Tensor>,
-        _decoder_input_ids: &Option<Tensor>,
+        _decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let output = match cache {
             Cache::ReformerCache(cached_layer_states) => self.forward_t(
-                input_ids.as_ref(),
+                input_ids,
                 None,
                 None,
-                attention_mask.as_ref(),
+                attention_mask,
                 None,
                 cached_layer_states,
                 train,
             ),
-            Cache::None => self.forward_t(
-                input_ids.as_ref(),
-                None,
-                None,
-                attention_mask.as_ref(),
-                None,
-                None,
-                train,
-            ),
+            Cache::None => self.forward_t(input_ids, None, None, attention_mask, None, None, train),
             _ => {
                 return Err(RustBertError::ValueError(
                     "Cache not compatible with Reformer Model".into(),
@@ -801,7 +786,7 @@ impl ReformerForSequenceClassification {
     ///
     /// let model_output = no_grad(|| {
     ///     reformer_model.forward_t(
-    ///         Some(&input_tensor),    
+    ///         Some(&input_tensor),
     ///         Some(&input_positions),
     ///         None,
     ///         Some(&attention_mask),
@@ -814,7 +799,7 @@ impl ReformerForSequenceClassification {
         &self,
         input_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         num_hashes: Option<i64>,
         train: bool,
@@ -939,7 +924,7 @@ impl ReformerForQuestionAnswering {
     ///
     /// let model_output = no_grad(|| {
     ///     reformer_model.forward_t(
-    ///         Some(&input_tensor),    
+    ///         Some(&input_tensor),
     ///         Some(&input_positions),
     ///         None,
     ///         Some(&attention_mask),
@@ -952,7 +937,7 @@ impl ReformerForQuestionAnswering {
         &self,
         input_ids: Option<&Tensor>,
         position_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         attention_mask: Option<&Tensor>,
         num_hashes: Option<i64>,
         train: bool,
@@ -972,8 +957,8 @@ impl ReformerForQuestionAnswering {
             .apply(&self.qa_outputs)
             .split(1, -1);
         let (start_logits, end_logits) = (&logits[0], &logits[1]);
-        let start_logits = start_logits.squeeze1(-1);
-        let end_logits = end_logits.squeeze1(-1);
+        let start_logits = start_logits.squeeze_dim(-1);
+        let end_logits = end_logits.squeeze_dim(-1);
 
         Ok(ReformerQuestionAnsweringModelOutput {
             start_logits,
@@ -1114,7 +1099,7 @@ impl PrivateLanguageGenerator<ReformerModelWithLMHead, ReformerVocab, ReformerTo
     fn get_model(&self) -> &ReformerModelWithLMHead {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
