@@ -1,15 +1,3 @@
-use std::borrow::Borrow;
-
-use rust_tokenizers::tokenizer::{T5Tokenizer, TruncationStrategy};
-use rust_tokenizers::vocab::T5Vocab;
-use serde::{Deserialize, Serialize};
-use tch::nn::embedding;
-use tch::{nn, Tensor};
-
-use crate::common::resources::{RemoteResource, Resource};
-use crate::gpt2::{Gpt2ConfigResources, Gpt2ModelResources, Gpt2VocabResources};
-use crate::pipelines::common::{ModelType, TokenizerOption};
-use crate::{Config, RustBertError};
 // Copyright 2018 Mesh TensorFlow authors, T5 Authors and HuggingFace Inc. team.
 // Copyright 2020 Guillaume Becquin
 // Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,14 +9,28 @@ use crate::{Config, RustBertError};
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+
+use std::borrow::Borrow;
+
+use rust_tokenizers::tokenizer::{T5Tokenizer, TruncationStrategy};
+use rust_tokenizers::vocab::T5Vocab;
+use serde::{Deserialize, Serialize};
+use tch::nn::embedding;
+use tch::{nn, Tensor};
+
+use crate::common::resources::{RemoteResource, Resource};
+use crate::gpt2::{Gpt2ConfigResources, Gpt2ModelResources, Gpt2VocabResources};
+use crate::pipelines::common::{ModelType, TokenizerOption};
 use crate::pipelines::generation_utils::private_generation_utils::{
     PreparedInput, PrivateLanguageGenerator,
 };
 use crate::pipelines::generation_utils::{
     Cache, GenerateConfig, LMHeadModel, LMModelOutput, LanguageGenerator,
 };
+use crate::pipelines::translation::Language;
 use crate::t5::attention::LayerState;
 use crate::t5::encoder::T5Stack;
+use crate::{Config, RustBertError};
 
 /// # T5 Pretrained model weight files
 pub struct T5ModelResources;
@@ -42,13 +44,19 @@ pub struct T5VocabResources;
 /// # T5 optional prefixes
 pub struct T5Prefix;
 
+/// # T5 source languages pre-sets
+pub struct T5SourceLanguages;
+
+/// # T5 target languages pre-sets
+pub type T5TargetLanguages = T5SourceLanguages;
+
 impl T5ModelResources {
-    /// Shared under Apache 2.0 license by the T5 Authors at https://github.com/google-research/text-to-text-transfer-transformer. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the T5 Authors at <https://github.com/google-research/text-to-text-transfer-transformer>. Modified with conversion to C-array format.
     pub const T5_SMALL: (&'static str, &'static str) = (
         "t5-small/model",
         "https://huggingface.co/t5-small/resolve/main/rust_model.ot",
     );
-    /// Shared under Apache 2.0 license by the T5 Authors at https://github.com/google-research/text-to-text-transfer-transformer. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the T5 Authors at <https://github.com/google-research/text-to-text-transfer-transformer>. Modified with conversion to C-array format.
     pub const T5_BASE: (&'static str, &'static str) = (
         "t5-base/model",
         "https://huggingface.co/t5-base/resolve/main/rust_model.ot",
@@ -56,12 +64,12 @@ impl T5ModelResources {
 }
 
 impl T5ConfigResources {
-    /// Shared under Apache 2.0 license by the Google team at https://github.com/google-research/text-to-text-transfer-transformer.
+    /// Shared under Apache 2.0 license by the Google team at <https://github.com/google-research/text-to-text-transfer-transformer>.
     pub const T5_SMALL: (&'static str, &'static str) = (
         "t5-small/config",
         "https://huggingface.co/t5-small/resolve/main/config.json",
     );
-    /// Shared under Apache 2.0 license by the Google team at https://github.com/google-research/text-to-text-transfer-transformer.
+    /// Shared under Apache 2.0 license by the Google team at <https://github.com/google-research/text-to-text-transfer-transformer>.
     pub const T5_BASE: (&'static str, &'static str) = (
         "t5-base/config",
         "https://huggingface.co/t5-base/resolve/main/config.json",
@@ -69,18 +77,25 @@ impl T5ConfigResources {
 }
 
 impl T5VocabResources {
-    /// Shared under Apache 2.0 license by the Google team at https://github.com/google-research/text-to-text-transfer-transformer.
+    /// Shared under Apache 2.0 license by the Google team at <https://github.com/google-research/text-to-text-transfer-transformer>.
     pub const T5_SMALL: (&'static str, &'static str) = (
         "t5-small/spiece",
         "https://huggingface.co/t5-small/resolve/main/spiece.model",
     );
-    /// Shared under Apache 2.0 license by the Google team at https://github.com/google-research/text-to-text-transfer-transformer.
+    /// Shared under Apache 2.0 license by the Google team at <https://github.com/google-research/text-to-text-transfer-transformer>.
     pub const T5_BASE: (&'static str, &'static str) = (
         "t5-base/spiece",
         "https://huggingface.co/t5-base/resolve/main/spiece.model",
     );
 }
 
+const T5LANGUAGES: [Language; 3] = [Language::English, Language::French, Language::German];
+
+impl T5SourceLanguages {
+    pub const T5_SMALL: [Language; 3] = T5LANGUAGES;
+    pub const T5_BASE: [Language; 3] = T5LANGUAGES;
+}
+
 impl T5Prefix {
     pub const ENGLISH2FRENCH: Option<&'static str> = Some("translate English to French:");
     pub const ENGLISH2GERMAN: Option<&'static str> = Some("translate English to German:");
@@ -99,7 +114,6 @@ pub struct T5Config {
     pub initializer_factor: f64,
     pub is_encoder_decoder: Option<bool>,
     pub layer_norm_epsilon: f64,
-    pub n_positions: i64,
     pub num_heads: i64,
     pub num_layers: i64,
     pub output_past: Option<bool>,
@@ -158,7 +172,7 @@ pub struct TranslationEnToRo {
     prefix: String,
 }
 
-impl Config<T5Config> for T5Config {}
+impl Config for T5Config {}
 
 /// # T5 Base model
 /// Base architecture for T5 model. Usually complemented with a task-specific head, such as a language model head.
@@ -315,8 +329,8 @@ impl T5Model {
         encoder_outputs: Option<&Tensor>,
         decoder_input_ids: Option<&Tensor>,
         decoder_attention_mask: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
-        decoder_input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
+        decoder_input_embeds: Option<&Tensor>,
         old_layer_states: Option<Vec<(Option<LayerState>, Option<LayerState>)>>,
         train: bool,
     ) -> T5ModelOutput {
@@ -450,7 +464,7 @@ impl T5ForConditionalGeneration {
     /// * `decoder_attention_mask` - Optional attention mask of shape (*batch size*, *target_sequence_length*) for the decoder positions. Positions with a mask with value 0 will be masked.
     /// * `input_embeds` - Optional input tensor of shape (*batch size*, *source_sequence_length*, *embeddings dimension*). This or `input_ids` must be provided.
     /// * `decoder_input_embeds` - Optional input tensor of shape (*batch size*, *target_sequence_length*, *embeddings dimension*). This or `decoder_input_ids` must be provided.
-    /// * `old_layer_states` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing th elast calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
+    /// * `old_layer_states` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing the last calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
     /// * `train` - boolean flag to turn on/off the dropout layers in the model. Should be set to false for inference.
     ///
     /// # Returns
@@ -507,8 +521,8 @@ impl T5ForConditionalGeneration {
         encoder_outputs: Option<&Tensor>,
         decoder_input_ids: Option<&Tensor>,
         decoder_attention_mask: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
-        decoder_input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
+        decoder_input_embeds: Option<&Tensor>,
         old_layer_states: Option<Vec<(Option<LayerState>, Option<LayerState>)>>,
         train: bool,
     ) -> T5ModelOutput {
@@ -558,7 +572,7 @@ impl LMHeadModel for T5ForConditionalGeneration {
     /// # Arguments
     ///
     /// * `input_ids` - Optional input tensor of shape (*batch size*, *sequence_length*). If None, pre-computed embeddings must be provided (see `input_embeds`)
-    /// * `layer_past` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing th elast calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
+    /// * `layer_past` - Optional vector of length `num_layers` containing tuples of optional `LayerStates` containing the last calculated key and value pairs for the decoder. This avoids recomputing attention weights at past positions and speeds up decoding.
     /// * `attention_mask` - Optional mask of shape (*batch size*, *sequence_length*). Masked position have value 0, non-masked value 1. If None set to 1
     /// * `input_embeds` - Unused for T5
     /// * `token_type_ids` - Unused for T5
@@ -612,22 +626,22 @@ impl LMHeadModel for T5ForConditionalGeneration {
     /// ```
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         cache: Cache,
-        attention_mask: &Option<Tensor>,
-        _token_type_ids: &Option<Tensor>,
-        _position_ids: &Option<Tensor>,
-        _input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        _token_type_ids: Option<&Tensor>,
+        _position_ids: Option<&Tensor>,
+        _input_embeds: Option<&Tensor>,
         encoder_outputs: Option<&Tensor>,
-        decoder_input_ids: &Option<Tensor>,
+        decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let base_model_output = match cache {
             Cache::T5Cache(cached_layer_states) => self.base_model.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
+                input_ids,
+                attention_mask,
                 encoder_outputs,
-                Option::from(decoder_input_ids),
+                decoder_input_ids,
                 None,
                 None,
                 None,
@@ -635,10 +649,10 @@ impl LMHeadModel for T5ForConditionalGeneration {
                 train,
             ),
             Cache::None => self.base_model.forward_t(
-                input_ids.as_ref(),
-                attention_mask.as_ref(),
+                input_ids,
+                attention_mask,
                 encoder_outputs,
-                Option::from(decoder_input_ids),
+                decoder_input_ids,
                 None,
                 None,
                 None,
@@ -778,7 +792,7 @@ impl PrivateLanguageGenerator<T5ForConditionalGeneration, T5Vocab, T5Tokenizer>
     fn get_model(&self) -> &T5ForConditionalGeneration {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
@@ -850,7 +864,7 @@ impl PrivateLanguageGenerator<T5ForConditionalGeneration, T5Vocab, T5Tokenizer>
     where
         S: AsRef<[&'a str]>,
     {
-        let tokens = self.get_tokenizer().encode_list(
+        let tokens = self._get_tokenizer().encode_list(
             prompt_text.as_ref(),
             max_len as usize,
             &TruncationStrategy::LongestFirst,
@@ -865,7 +879,7 @@ impl PrivateLanguageGenerator<T5ForConditionalGeneration, T5Vocab, T5Tokenizer>
 
         let pad_token = match pad_token_id {
             Some(value) => value,
-            None => self.get_tokenizer().get_unk_id(),
+            None => self._get_tokenizer().get_unk_id(),
         };
 
         let token_ids = token_ids
