@@ -43,7 +43,7 @@ pub struct XLNetConfigResources;
 pub struct XLNetVocabResources;
 
 impl XLNetModelResources {
-    /// Shared under Apache 2.0 license by the XLNet Authors at https://github.com/zihangdai/xlnet. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the XLNet Authors at <https://github.com/zihangdai/xlnet>. Modified with conversion to C-array format.
     pub const XLNET_BASE_CASED: (&'static str, &'static str) = (
         "xlnet-base-cased/model",
         "https://huggingface.co/xlnet-base-cased/resolve/main/rust_model.ot",
@@ -51,7 +51,7 @@ impl XLNetModelResources {
 }
 
 impl XLNetConfigResources {
-    /// Shared under Apache 2.0 license by the XLNet Authors at https://github.com/zihangdai/xlnet. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the XLNet Authors at <https://github.com/zihangdai/xlnet>. Modified with conversion to C-array format.
     pub const XLNET_BASE_CASED: (&'static str, &'static str) = (
         "xlnet-base-cased/config",
         "https://huggingface.co/xlnet-base-cased/resolve/main/config.json",
@@ -59,7 +59,7 @@ impl XLNetConfigResources {
 }
 
 impl XLNetVocabResources {
-    /// Shared under Apache 2.0 license by the XLNet Authors at https://github.com/zihangdai/xlnet. Modified with conversion to C-array format.
+    /// Shared under Apache 2.0 license by the XLNet Authors at <https://github.com/zihangdai/xlnet>. Modified with conversion to C-array format.
     pub const XLNET_BASE_CASED: (&'static str, &'static str) = (
         "xlnet-base-cased/spiece",
         "https://huggingface.co/xlnet-base-cased/resolve/main/spiece.model",
@@ -116,7 +116,7 @@ pub struct XLNetConfig {
     pub chunk_size_feed_forward: Option<i64>,
 }
 
-impl Config<XLNetConfig> for XLNetConfig {}
+impl Config for XLNetConfig {}
 
 /// # XLNet Base model
 /// Base architecture for XLNet models. Task-specific models will be built from this common base model
@@ -287,13 +287,16 @@ impl XLNetModel {
         batch_size: Option<i64>,
         device: Device,
     ) -> Tensor {
-        let frequency_sequence = Tensor::arange2(0, self.d_model, 2, (Kind::Float, device));
-        let inverse_frequency = 1f64 / Tensor::pow2(10000f64, &(frequency_sequence / self.d_model));
+        let frequency_sequence =
+            Tensor::arange_start_step(0, self.d_model, 2, (Kind::Float, device));
+        let inverse_frequency =
+            1f64 / Tensor::pow_scalar(10000f64, &(frequency_sequence / self.d_model));
         let (begin, end) = match self.attention_type {
             AttentionType::bi => (k_len, -q_len),
             AttentionType::uni => (k_len, -1),
         };
-        let mut forward_positions_sequence = Tensor::arange2(begin, end, -1, (Kind::Float, device));
+        let mut forward_positions_sequence =
+            Tensor::arange_start_step(begin, end, -1, (Kind::Float, device));
         match self.clamp_len {
             Some(clamp_value) if clamp_value > 0 => {
                 let _ = forward_positions_sequence.clamp_(-clamp_value, clamp_value);
@@ -302,7 +305,7 @@ impl XLNetModel {
         }
         if self.bi_data {
             let mut backward_positions_sequence =
-                Tensor::arange2(-begin, -end, 1, (Kind::Float, device));
+                Tensor::arange_start(-begin, -end, (Kind::Float, device));
             match self.clamp_len {
                 Some(clamp_value) if clamp_value > 0 => {
                     let _ = backward_positions_sequence.clamp_(-clamp_value, clamp_value);
@@ -387,38 +390,34 @@ impl XLNetModel {
         perm_mask: Option<&Tensor>,
         target_mapping: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> Result<XLNetModelOutput, RustBertError> {
-        let (word_emb_k, input_shape) = match input_ids {
-            Some(input_value) => match input_embeds {
-                Some(_) => {
-                    return Err(RustBertError::ValueError(
-                        "Only one of input ids or input embeddings may be set".into(),
-                    ));
-                }
-                None => {
-                    let size = input_value.size();
-                    (
-                        input_value
-                            .transpose(0, 1)
-                            .contiguous()
-                            .apply_t(&self.word_embeddings, train),
-                        vec![size[1], size[0]],
-                    )
-                }
-            },
-            None => match input_embeds {
-                Some(embeds) => {
-                    let size = vec![embeds.size()[1], embeds.size()[0]];
-                    (embeds.transpose(0, 1).contiguous(), size)
-                }
-                None => {
-                    return Err(RustBertError::ValueError(
-                        "At least one of input ids or input embeddings must be set".into(),
-                    ));
-                }
-            },
+        let (word_emb_k, input_shape) = match (input_ids, input_embeds) {
+            (Some(_), Some(_)) => {
+                return Err(RustBertError::ValueError(
+                    "Only one of input ids or input embeddings may be set".into(),
+                ));
+            }
+            (Some(input_value), None) => {
+                let size = input_value.size();
+                (
+                    input_value
+                        .transpose(0, 1)
+                        .contiguous()
+                        .apply_t(&self.word_embeddings, train),
+                    vec![size[1], size[0]],
+                )
+            }
+            (None, Some(embeds)) => {
+                let size = vec![embeds.size()[1], embeds.size()[0]];
+                (embeds.transpose(0, 1).contiguous(), size)
+            }
+            (None, None) => {
+                return Err(RustBertError::ValueError(
+                    "At least one of input ids or input embeddings must be set".into(),
+                ));
+            }
         };
 
         let token_type_ids =
@@ -512,7 +511,7 @@ impl XLNetModel {
             };
             let seg_mat = token_type_ids_value
                 .unsqueeze(-1)
-                .ne1(&cat_ids.unsqueeze(0))
+                .ne_tensor(&cat_ids.unsqueeze(0))
                 .to_kind(Kind::Int64);
             Some(seg_mat.one_hot(2).to_kind(Kind::Float))
         } else {
@@ -712,7 +711,7 @@ impl XLNetLMHeadModel {
         perm_mask: Option<&Tensor>,
         target_mapping: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         let base_model_output = self.base_model.forward_t(
@@ -791,35 +790,35 @@ impl LMHeadModel for XLNetLMHeadModel {
     /// ```
     fn forward_t(
         &self,
-        input_ids: &Option<Tensor>,
+        input_ids: Option<&Tensor>,
         layer_past: Cache,
-        attention_mask: &Option<Tensor>,
-        _token_type_ids: &Option<Tensor>,
-        _position_ids: &Option<Tensor>,
-        _input_embeds: &Option<Tensor>,
+        attention_mask: Option<&Tensor>,
+        _token_type_ids: Option<&Tensor>,
+        _position_ids: Option<&Tensor>,
+        _input_embeds: Option<&Tensor>,
         _encoder_outputs: Option<&Tensor>,
-        decoder_input_ids: &Option<Tensor>,
+        decoder_input_ids: Option<&Tensor>,
         train: bool,
     ) -> Result<LMModelOutput, RustBertError> {
         match layer_past {
             Cache::XLNetCache(layer_past) => self.forward_t(
-                input_ids.as_ref(),
+                input_ids,
                 None,
                 layer_past,
-                attention_mask.as_ref(),
+                attention_mask,
                 // For XLNet the decoder_input_ids are used as a placeholder for the target mapping
-                decoder_input_ids.as_ref(),
+                decoder_input_ids,
                 None,
                 None,
                 train,
             ),
             Cache::None => self.forward_t(
-                input_ids.as_ref(),
+                input_ids,
                 None,
                 None,
-                attention_mask.as_ref(),
+                attention_mask,
                 // For XLNet the decoder_input_ids are used as a placeholder for the target mapping
-                decoder_input_ids.as_ref(),
+                decoder_input_ids,
                 None,
                 None,
                 train,
@@ -963,7 +962,7 @@ impl XLNetForSequenceClassification {
         perm_mask: Option<&Tensor>,
         target_mapping: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> XLNetSequenceClassificationOutput {
         let base_model_output = self
@@ -1121,7 +1120,7 @@ impl XLNetForTokenClassification {
         perm_mask: Option<&Tensor>,
         target_mapping: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> XLNetTokenClassificationOutput {
         let base_model_output = self
@@ -1270,7 +1269,7 @@ impl XLNetForMultipleChoice {
         perm_mask: Option<&Tensor>,
         target_mapping: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> XLNetSequenceClassificationOutput {
         let (input_ids, num_choices) = match input_ids {
@@ -1302,7 +1301,7 @@ impl XLNetForMultipleChoice {
                 perm_mask,
                 target_mapping,
                 token_type_ids.as_ref(),
-                input_embeds,
+                input_embeds.as_ref(),
                 train,
             )
             .unwrap();
@@ -1441,7 +1440,7 @@ impl XLNetForQuestionAnswering {
         perm_mask: Option<&Tensor>,
         target_mapping: Option<&Tensor>,
         token_type_ids: Option<&Tensor>,
-        input_embeds: Option<Tensor>,
+        input_embeds: Option<&Tensor>,
         train: bool,
     ) -> XLNetQuestionAnsweringOutput {
         let base_model_output = self
@@ -1461,8 +1460,8 @@ impl XLNetForQuestionAnswering {
         let sequence_output = base_model_output.hidden_state.apply(&self.qa_outputs);
         let logits = sequence_output.split(1, -1);
         let (start_logits, end_logits) = (&logits[0], &logits[1]);
-        let start_logits = start_logits.squeeze1(-1);
-        let end_logits = end_logits.squeeze1(-1);
+        let start_logits = start_logits.squeeze_dim(-1);
+        let end_logits = end_logits.squeeze_dim(-1);
 
         XLNetQuestionAnsweringOutput {
             start_logits,
@@ -1615,7 +1614,7 @@ impl PrivateLanguageGenerator<XLNetLMHeadModel, XLNetVocab, XLNetTokenizer> for
     fn get_model(&self) -> &XLNetLMHeadModel {
         &self.model
     }
-    fn get_tokenizer(&self) -> &TokenizerOption {
+    fn _get_tokenizer(&self) -> &TokenizerOption {
         &self.tokenizer
     }
     fn get_var_store(&self) -> &nn::VarStore {
